dojo.provide("apiviewer.apiModule");
// dojo.require("dojox.data.jsonPathStore");
dojo.require("toolbox.utils");
dojo.require("dojox.storage.AirFileStorageProvider");
dojo.require("toolbox.Module");

air.trace("apiModule.js");
(function() {
	/*
		
	*/
	var History = function() {
		// summary: 
		//  basically just keep an array of functions,
		//  and moves a pointer in the array
		this.currentIndex = -1;
		this._states = [];
		this.length = 0;
	};
	dojo.extend(History, {
		add: function(callback) {
			// add is always to the end of the array
			// so first truncate the array
			if(this.currentIndex < this.length-1) {
				this._truncate();
			}
			this._states.push(callback);
			this.length = this._states.length;
			this.currentIndex = this.length-1;
		},
		
		_truncate: function(len) {
			if(Number(len)) {
				this.currentIndex = Math.min(len-1, this.currentIndex); 
			} else {
				len = this.currentIndex+1; 
			}
			this.length = this._states.length = len;
			//console.log("truncating history to len:" + len + ", with result: " + this._states.length);
		},
		back: function() {
			if(this.currentIndex > 0) {
				this.go(this.currentIndex-1);
			}
		},
		forward: function() {
			if(this.currentIndex < this.length-1) {
				this.go(this.currentIndex+1);
			}
		},
		go: function(idx) {
			//console.log("History.go, called with idx: " + idx);
			var callback = this._states[idx];
			if(callback) {
				this.currentIndex = idx;
				callback();
			} else {
				//console.log("History: no entry for " + idx);
			}
		}
	});
	
	window.apiModule = new toolbox.Module("apiviewer", {

		moduleDirectory: air.File.applicationDirectory.resolvePath("modules/apiviewer"),
		storageDirectory: "apiviewer",

		topics: {
			LOADING_DOCS: "/api/loadingdocs", 
			END_LOADING_DOCS: "/api/endloadingdocs", 
			BUSY: "/apiviewer/busy",
			END_BUSY: "/apiviewer/endbusy",
			PREPARING_UI: "/apiviewer/prepareui",
			END_PREPARING_UI: "/apiviewer/endprepareui",
			ON_CHILD_LOAD: "/apiviewer/onchildload"
		},
		_cssNeedsRefresh: false,
		_pendingHistoryIndex: null,
		
		init:function() {
			this.childHistory = new History();
			// console.log("zip is at url: " + this.zipUrl);

			// set up the sandbox bridge
			this.sandboxInterface = {
				onChildLoad: dojo.hitch(this, function(evt) {
					dojo.publish(this.topics.ON_CHILD_LOAD, [evt]);
				}), 
				searchFor: dojo.hitch(this, this.searchFor), 
				getSearchResultsHTML: dojo.hitch(this, this.getSearchResultsHTML) 
			};

			var deferred = new dojo.Deferred(); 
			deferred.addCallback(dojo.hitch(this, 
				function() {

					// copy over our resources directory
					// it provides/ & overrides in places the search results page, images, css and script for the api documents themselves
					this._monkeypatchAPIResources();
					
					dojo.publish(this.topics.END_LOADING_DOCS, []);

					dojo.publish(this.topics.PREPARING_UI, []);
			
					dojo.connect(dojo.byId("searchtext"), "onkeypress", apiModule, 
						function(e){
							if(e.keyCode == 13){
								this.loadSearchPage();
							}
						}
					);
					this._initNav();
				}
			));
			deferred.addErrback(dojo.hitch(this, 
				function(err) {
					dojo.publish(this.topics.END_LOADING_DOCS, []);
					console.warn("error from _updateAPIDocs:" + err);
				}
			));

			dojo.subscribe(this.topics.LOADING_DOCS, this, function(evt) {
				this.showActivity(evt.message);
			});
			dojo.subscribe(this.topics.END_LOADING_DOCS, this, function(evt) {
				// remove the dialog
				this.hideActivity();
			});

			dojo.subscribe(this.topics.PREPARING_UI, this, function() {
				this.showActivity("Preparing the viewer...");
			});
			dojo.subscribe(this.topics.END_PREPARING_UI, this, function(evt) {
				if(this._cssNeedsRefresh) {
					toolbox.utils.reCSS();
				}
				// load up the default page
				this.loadDoc(this.defaultDocumentUrl);
				this.currentDocumentUrl = this.defaultDocumentUrl;
				
				// now the ui is presentable, show it
				dojo.style("page-block", "visibility", "visible");
				setTimeout(dojo.hitch(this, function() {
					// re-layout the nav
					dijit.byId("navigation-block").resize();
					this.hideActivity();
				}), 50);
			});

			dojo.subscribe(this.topics.ON_CHILD_LOAD, this, function(evt) {
				var bridge = evt.contentWindow.childSandboxBridge;
				var defaultDocumentUrl = this.currentDocumentUrl = bridge.location;
				// sync the nav with the new document
				this._syncNav();
				
				this._updateHistory(bridge.location);
			});
			
			dojo.publish(this.topics.LOADING_DOCS, [{ message: "Loading API documents..." }]);
			this._updateAPIDocs(this.zipUrl, deferred); 
			
		},
		_syncNav: function(currentDocumentUrl) {
			// TODO: open the right panel on the nav if necessary
			var leftAccordion = dijit.byId("navigation-block");
			var docUrl = currentDocumentUrl || this.currentDocumentUrl; 
			if(! (leftAccordion && docUrl)) {
				// bail if the nav isnt initalized yet
				console.log("_syncNav didnt have leftAccordion && docUrl")
				return; 
			}
			var apiUrl = this.storageDirectory.resolvePath("api/api").url;

			if(docUrl.indexOf(apiUrl) == 0) {
				var part = docUrl.substring(apiUrl.length);
				var match = part.match(/nodes\/([^\/]+)/); 
				if(match) {
					this.currentDocumentNS = match[1];
				}
			}

			var title = dojo.attr(leftAccordion.selectedChildWidget.domNode, "title");
			var leftnavNS = title.toLowerCase();
			
			if(this.currentDocumentNS !== leftnavNS && dijit.byId(this.currentDocumentNS+"ProjectPane")) {
				leftAccordion.selectChild(this.currentDocumentNS+"ProjectPane");
			}
		},
		_updateHistory: function(childUrl) {
			//console.log("_updateHistory got: " + childUrl + ", pendingIndex: " + this._pendingHistoryIndex);
			if(typeof this._pendingHistoryIndex == "number") {
				//console.log("back/forward event, no new history entry");
				delete this._pendingHistoryIndex;
			} else {
				// add the new entry
				this.childHistory.add(dojo.hitch(this, function() {
					//console.log("history entry: " + childUrl);
					documentPane.setUrl(childUrl);
				}));
			}
		},
		historyForward: function() {
			var lastIndex = this.childHistory.length-1; 
			var currIndex = this.childHistory.currentIndex; 
			// we set a flag so that the resulting load event doesnt result in a new history entry
			this._pendingHistoryIndex = (currIndex < lastIndex) ? currIndex+1 : lastIndex; 
			this.childHistory.forward();
		},
		historyBack: function() {
			var currIndex = this.childHistory.currentIndex; 
			// we set a flag so that the resulting load event doesnt result in a new history entry
			this._pendingHistoryIndex = (currIndex > 0) ? currIndex-1 : 0; 
			this.childHistory.back();
		},
		_checkDirty: function(deferred) {
			// TODO: implement check of file age vs. what we have stored
			deferred.callback(true);
			return deferred;
		},
		_updateAPIDocs: function(url, deferred) {
			var url = url || this.zipUrl; 
			
			// TODO: include version in directory name? 
			var apiDir = this.storageDirectory.resolvePath("api");
			// console.log("apiDir is at: " + apiDir.nativePath);
			
			var prefs = new dojox.storage.AirFileStorageProvider();
			
			if(apiDir.exists) {
				//console.log("apiDir exists");
				var apiversion = prefs.get("version", "apiviewer");
				if(apiversion && apiversion == url){
					console.log("docs up to date");
					return deferred.callback();
				} else {
					console.log("Stale docs in " + apiDir.nativePath + ". Deleting");
					apiDir.deleteDirectory(true);
				}
			}
			// we're going to need to re-fetch css that didnt exist
			// when the page first rendered
			this._cssNeedsRefresh = true;

			// The api directory doesnt exist, fetch and expand the zip file
			
			// TODO: version numbers in filename? 
			var zipFile = this.storageDirectory.resolvePath("api.zip");
			var zipHandler = new toolbox.utils.Zip();
			
			// console.log("priming the zipHandler with:"+ zipFile.nativePath+ ", exists? " + zipFile.exists);
			if(zipFile.exists) {
				zipHandler.setFile( zipFile );
			} else {
				zipHandler.setUrl( this.zipUrl );
			}

			var activityWidget = dijit.byId("activityBox");
			activityWidget.setMessage("API Docs need updating. Downloading...");

			// keep some state, progress information for the event handlers
			var docsUpdate = this._docsUpdate = {
				stateName: "downloading", 
				onProgress: function(evt){
					if(evt.bytesLoaded / evt.bytesTotal > 0.9){
						activityWidget.setMessage("Loading resources...");
						activityWidget.update({indeterminate: true, maximum: 100, places:0,
							progress: "0"});
						dojo.disconnect(onProgressConnect);
					} else {
						activityWidget.update({indeterminate: false,
							maximum: evt.bytesTotal,
							progress: evt.bytesLoaded});
					}
				}, 
				onFileLoaded: function(evt){
					if(docsUpdate.stateName == "downloading") {
						// this is the first fileloaded event
						// todo: we can get a filecount, and give progress
						this._docsUpdate.stateName = "expanding";
					}
					var filename = evt.file.filename; 
					var startIdx = (filename.indexOf("api/api") > -1) ? filename.indexOf("api/api") + "api/api".length : 0; 
					activityWidget.setMessage("Unpacked:" + filename.substring( startIdx ));
				},
				onComplete: function() {
					// sanity check
					// kill the fallback timer
					// clearInterval(this._docsCheckTimer);
					if(apiDir.exists) {
						prefs.put("version", url, null, "apiviewer");
						deferred.callback();
					} else {
						deferred.errback("fetched and unpacked api.zip, but its still not at " + apiDir.nativePath);
					}
					delete this._docsUpdate;
				},
				onError: function(evt) {
					//console.log("publishing " + this.topics.END_LOADING_DOCS);
					dojo.publish(
						this.topics.END_LOADING_DOCS, 
						[
							dojo.mixin(evt, {
								message: "Error downloading and unpacking zip file"
							})
						]
					);
					delete this._docsUpdate;
					deferred.errback(
						"Error downloading and unpacking zip file to	" + apiDir.nativePath
						+ (evt.message || "")
					);
				}
			}; 
			
			var onCompleteConnect = 	dojo.connect(zipHandler, "onLoadComplete", this, docsUpdate.onComplete);
			var onErrorConnect = 			dojo.connect(zipHandler, "onError", this, docsUpdate.onError);
			var onProgressConnect = 	dojo.connect(zipHandler, "onProgress", this, docsUpdate.onProgress);
			var onFileLoadedConnect = dojo.connect(zipHandler, "onFileLoaded", this, docsUpdate.onFileLoaded);

			// do it
			zipHandler.expandTo( this.storageDirectory );
		},
		_initNav: function() {
			var xmlUrl = this.storageDirectory.resolvePath("./api/index.xml").url;
			var xslUrl = this.navXslUrl;

			var def = dojo.xhrGet({
				url: xmlUrl, 
				handleAs: "text"
			}); 
			var xml, xsl;
			var self = this;
			def.addCallback(dojo.hitch(this, function(str) {
				console.log("loaded xml str: " + str.substring(0, 100));
				xml = str;
				var xslDefd = dojo.xhrGet({
					url: xslUrl, 
					handleAs: "text"
				});
				xslDefd.addCallback(
					dojo.hitch(this, function(str) {
						console.log("loaded xsl str: " + str.substring(0, 100));
						xsl = str;
						
						var navWidget = dijit.byId("navigation-block");
						// push the result of the xsl transform into a temporary offscreen container
						var tmpContainer = dojo.doc.createElement("div"); 
						tmpContainer.style.cssText = "position:absolute; width: 200px; left: -200px";
						dojo.body().appendChild(tmpContainer); 
						
						// push the new nav content into the nav widget (currently an accordioncontainer) 
						// parse and add the new widgets as children
						console.log("setting new content for " + navWidget.id);
						
						dojo.html.set(
							tmpContainer, 
							toolbox.utils.transformXML(xml, xsl), 
							{ 
								parseContent: true,
								onEnd: function() {
									dojo.html._ContentSetter.prototype.onEnd.call(this);
									console.log("in onEnd, we created: " + this.parseResults.length + " widgets");
									
									dojo.forEach(this.parseResults, function(cwidget) {
										navWidget.addChild(cwidget);
									});
								}
							}
						);
						// cleanup: remove the tmp container
						dojo.body().removeChild(tmpContainer);
					})
				);
				xslDefd.addCallback(
					dojo.hitch(this, function(str) {
						setTimeout(dojo.hitch(this, function() {
							dojo.publish(this.topics.END_PREPARING_UI, []);
						}) ,0)
					})
				);
			}));
			def.addErrback(dojo.hitch(this, function(err) {
				this.showFatalError(err); 
				dojo.publish(this.topics.END_PREPARING_UI, []);
			}))
		},
		_onDocClick: function(evt) {
			var node = evt.target;
			if(node.tagName !== "A") {
				return;
			}
			var url = dojo.attr(node, "href");
			if(url.indexOf("http") == 0) {
				dojo.stopEvent(evt);
				// handle as external link
			} else {
			}
		},
		_onNavClick: function(evt) {
			var fixUrl = function(url) {
				url = url.replace( /^docs/, "api" );
				return url;
			};
			
			var node = evt.target;

			dojo.stopEvent(evt);

			if(node.tagName == "A") {
					var url = fixUrl(dojo.attr(node, "href"));
					console.log("loading doc at: " + url);
					this.loadDoc(url);
			}
		},
		
		loadDoc: function(url) {
			// setUrl on the iframe widget
			dijit.byId("content-block").setUrl(url);
		},
		getSearchResultsHTML: function() {
			return this._searchResultsHTML; 
		},
		loadSearchPage : function() {
			// search/search result sequence: 
			// clicking search, calls loadSearchPage
			// this loads the searchresult.html into the iframe, puting the term in the querystring
			// when that loads, it kicks off the actual search with searchFor(term);
			// the results are stashed on this._searchResultsHTML
			// the iframe asks for the result periodically, and displays them when done
			
			// clean out the last results
			delete this._searchResultsHTML; 

			var resultUrl = this.storageDirectory.resolvePath("api/api/searchresult.html").url;
			var term = dojo.byId("searchtext").value; 
			resultUrl += "?term=" + term; 
			
			console.log("loading search page");
			documentPane.setUrl(resultUrl);
			
			// listen for the load event, then disconnect
			// var deferred = new dojo.Deferred();
			// 
			// deferred.addCallback(dojo.hitch(this, function(evt) {
			// 	// the result page is loaded, now to put some results in it
			// 	var bridge = evt.contentWindow.childSandboxBridge;
			// 	
			// 	console.log("On search page load, bridge is: " + bridge);
			// 	console.log("child resultTester: " + bridge.resultTester);
			// 	console.log("child title: " + bridge.title);
			// 	// bridge.updateHeader("Ah Ha!!");
			// 	// 				bridge.addResultLine({ title: "My Test Result Item", filename: "nodes/foo.html"});
			// }));
			// var handle; 
			// handle = dojo.subscribe(this.topics.ON_CHILD_LOAD, this, function(evt) {
			// 	console.log("On search page load, invoking the callback");
			// 	deferred.callback(evt);
			// 	console.log("dojo.unsubscribe with handle: " + handle);
			// 	dojo.unsubscribe(handle);
			// }); 
		},

		searchFor: function(query) {
			console.log("searchFor: " + query);
			if(!query){
				//query = dojo.byId('searchtext').value;
				console.log("TODO: handle no search term");
				return;
			}
			if(query.substring(0,1) == "?"){
				query = query.substring(1);
			}
			
			query = unescape(query);

			// create the required interface elements
			// we build the html in an offscreen container 
			// then make the innerHTML available to the iframe
			var tmpContainer = document.createElement("div");
			var resultList = document.createElement("ul");
			resultList.id = "search-results";
			var status = document.createElement("div");
			status.id = "status";
			tmpContainer.appendChild(status);
			tmpContainer.appendChild(resultList);

			// stem the searchwords and add them to the
			// correct list
			var stemmer = new PorterStemmer();
			var searchwords = [];
			var excluded = [];
			var hlwords = [];
			var tmp = query.split(/\s+/);
			for (var i = 0; i < tmp.length; i++) {
					var word = tmp[i];
					// stem the word
					if(word.indexOf(".") == -1){
						word = stemmer.stemWord(word);
					}
					word = word.toLowerCase();
					// select the correct list
					if (word[0] == '-') {
							var toAppend = excluded;
							word = word.substr(1);
					}
					else {
							var toAppend = searchwords;
							hlwords.push(tmp[i].toLowerCase());
					}
					// only add if not already in the list
					if (dojo.indexOf(toAppend, word) == -1) {
							toAppend.push(word);
					}
			};
			// no search highlighting for now
					//var highlightstring = '?highlight=' + $.urlencode(hlwords.join(" "));

			var databaseFile = this.storageDirectory.resolvePath("api/searchindex.db");
			if (!databaseFile.exists) {
				// this should not happen
				this.showFatalError("The search index is missing. Unable to search.");
				return;
			}

			var conn = new air.SQLConnection();
			conn.open(databaseFile);
			var stmt = new air.SQLStatement();
			var query = "select filename, title, " +
				"sum(titlecount)*10+sum(totalcount) as score from " +
				"word_matches, files where files.id=word_matches.file_id " +
				"and word_matches.word_id in (select id from words where ";
			for(var i = 0; i < searchwords.length; i++){
				if(i>0){
					query += " or ";
				}
				query += "word = :word" + i;
			}
			query += ")";
			if(excluded.length > 0){
				query += " and word_matches.file_id not in (select file_id " +
					"from words, word_matches where word_matches.word_id = " +
					"words.id and ";
				for(i = 0; i < excluded.length; i++){
					if(i>0){
						query += " or ";
					}
					query += "word = :eword" + i;
				}
				query += ")";
			}
			query += " group by filename order by score";
			// Note order by score means that the query results will
			// be presented with the lowest score first. However,
			// displayNextItem below uses "appendChild", which means
			// that the ordering is automatically reversed
			console.debug("Query: " + query);

			stmt.text = query;
			for(i = 0; i < searchwords.length; i++){
				stmt.parameters[":word" + i] = searchwords[i];
			}
			for(i = 0; i < excluded.length; i++){
				stmt.parameters[":eword" + i] = excluded[i];
			}
			stmt.sqlConnection = conn;
			this.showActivity("Loading search index");
			stmt.execute();
			var results = stmt.getResult();
			results = results.data;

			// print the results
			var resultCount = results ? results.length : 0;
			var self = this;

			function displayNextItem() {
				// results left, load the summary and display it
				if(results && results.length) {
					var item = results.pop();
					var evenodd = results.length % 2 ? "even" : "odd";
					var listItem = document.createElement("li");
					dojo.addClass(listItem, evenodd);
					var link = document.createElement("A");
					listItem.appendChild(link);
					dojo.attr(link, "href", item.filename); // "api/api/" 
					link.innerHTML = item.title;
					//dojo.connect(link, "onclick", apiModule, 
					//			apiModule._onNavClick);
					resultList.appendChild(listItem);
					displayNextItem();
				}
				// search finished, update title and status message
				else {
						//pulseStatus = -1;
						//title.innerHTML = 'Search Results';
						if (!resultCount) {
								status.innerHTML = 'Your search did not match any documents. ' +
														'Please make sure that all words are spelled ' +
														'correctly. Note that very common words are ' +
														'excluded from the search index.';
						}
						else {
								status.innerHTML ='Search finished, found ' + resultCount +
														' item' + (resultCount != 1 ? 's' : '');
						}
						dojo.style(status, "display", "block");

						// make the results available as an html string 
						self._searchResultsHTML = tmpContainer.innerHTML; 

						// cleanup
						tmpContainer.innerHTML = ""; 
						tmpContainer = null;

						conn.close();
						self.hideActivity();
					}
			}
			displayNextItem();
		},
		
		_monkeypatchAPIResources: function() {
			console.log("calling _monkeypatchAPIResources");
			var targetDir = this.storageDirectory.resolvePath("api/api");
			toolbox.utils.mergeDirectories(this.documentPaneResourcesDirectory, targetDir);
			console.log("done calling _monkeypatchAPIResources");
		}
		
	});
	
	// mixin some constants
	air.trace("mixing in my constants, apiModule.storageDirectory is: " + apiModule.storageDirectory.url);
	dojo.mixin(apiModule, {
		zipUrl: "http://download.dojotoolkit.org/toolbox/api/api-1_1d.zip",
		navXslUrl: apiModule.moduleDirectory.resolvePath("index.xsl").url,
		documentPaneResourcesDirectory: apiModule.moduleDirectory.resolvePath("content_resources"),
		defaultDocumentUrl: apiModule.storageDirectory.resolvePath("api/api/nodes/dojo/dojo.html").url
	});
	air.trace("done, mixing in my constants, defaultDocumentUrl is: " + apiModule.defaultDocumentUrl);
})();
air.trace("end of apiModule.js");
