dojo.provide("builder.airbuild");
// dojo.require("dojox.storage.AirDBStorageProvider")

dojo.require("toolbox.Module");
dojo.require("dojo.html");
dojo.require("toolbox.MessageDialog");
dojo.require("toolbox.ShrinkSafeRequest");
dojo.require("dijit.Tooltip");

dojo.require("dojox.lang.functional.object");
// convenience alias
dojo.forIn = dojox.lang.functional.forIn; 

dojo.require("builder.buildoptions");

(function() {

	window.builderModule = builder.airbuild = new toolbox.Module("builder", {
		// set the working directory for this module
		moduleDirectory: air.File.applicationDirectory.resolvePath("modules/builder"),
		
		init: function() {
			// summary: 
			// 	init UI
			// description: 
			// 	hook up the tooltips
			// 	and any other work before the UI becomes interactive
			this._initialized = true;
			
			// TODO: develop and utilize a standard help-display system in toolbox.Module
			var helpClickHandler = dojo.hitch(this, this._onHelpButtonClick);
			dojo.query(".helpButton", dojo.byId("main")).onclick(helpClickHandler);
			if(window.commandLine && window.commandLine.length > 0){
				air.trace("Builder command line: " + window.commandLine);
				this.runbuild(this.convertArrayToObject(commandLine));
			}
		},
		
		convertArrayToObject : function(/*Array*/ary){
			//summary: converts an array that has String members of "name=value"
			//into an object, where the properties on the object are the names in the array
			//member name/value pairs.
			//originally in util/buildscripts/jslib/buildUtil.js
			var result = {};
			for(var i = 0; i < ary.length; i++){
				var separatorIndex = ary[i].indexOf("=");
				if(separatorIndex == -1){
					throw "Malformed name/value pair: [" + ary[i] + "]. Format should be name=value";
				}
				result[ary[i].substring(0, separatorIndex)] = ary[i].substring(separatorIndex + 1, ary[i].length);
			}
			return result; //Object
		},

		_onHelpButtonClick: function(evt) {
			var btnNode = evt.target, 
				term = dojo.attr(btnNode, "term");

			evt.preventDefault();
			if(!term) {
				console.warn("no term associated with clicked element");
				return;
			}
			// populate the dialog using helpText from the buildoptions module
			var text= builder.buildoptions[term].helpText; 
			if(!text) {
				console.warn("no help text available for term: " + term);
				return;
			}
			text= text.replace("\n", "<br/>"); 

			// show it
			// TODO: this is a pita to style and control, needs another implementation when 
			// were clearer about what we want from help here
			dijit.showTooltip(text, btnNode);
			
			// as the open is async, we defer registering the onclick handler to cancel
			setTimeout(dojo.hitch(this, function() {
				this._isHelpOpen = true;
				this._closeHelpHandle = dojo.connect(dojo.body(), "onclick", dojo.hitch(this, function() {
					if(this._isHelpOpen) {
						dijit.hideTooltip(btnNode);
						this._isHelpOpen = false;
						dojo.disconnect(this._closeHelpHandle);
					}
				}));
			}), 800);
				
		},
		
		getConfigFromUI: function(form) {
			// summary: 
			// 	gets values from the form and returns a property bag of the values
			
			// this helps us track which options the UI is providing vs. the rhino shell version
			var utilBuildOptions = dojo.mixin({}, builder.buildoptions);
			
			if(form) {
				var formData = dojo.formToObject(form);
				dojo.query("[widgetid]", form).forEach(function(elm) {
					var widget = dijit.byNode(elm);
					if(widget && typeof widget["getValue"] == "function") {
						console.log("adding form widget value: " + widget.id + "=" + widget.getValue());
						formData[widget.id] = widget.getValue();
					}
				});
				
				dojo.forIn(formData, function(value, key) {
					var refObject = utilBuildOptions[key];
					if(typeof refObject !== "undefined") {
						console.log(key + " found in build options, with value: " + value);
						
						var defaultValue = refObject.defaultValue; 
						var isRequired = refObject.required; 
						if(typeof defaultValue == "boolean") {
							defaultValue = defaultValue.toString();
						}
						
						if(isRequired) {
							if(!value) {
								formData[key] = defaultValue || ""; 
							}
						} else {
							if((typeof defaultValue !== "undefined" && value) && defaultValue == value) {
								// simplify the cmdline by removing non-required fields submitted with their default values
								delete formData[key];
							}
						}
						delete utilBuildOptions[key];
						console.log(key + " has value: " + formData[key]);
					} else {
						delete formData[key];
						console.log("junk dataa: " + key + " found in form data but NOT in build options");
					}
				});
				
				for(var key in utilBuildOptions) {
					if(utilBuildOptions[key].required && !(key in formData)) {
						console.log("adding required field: " + key + " has value: " + formData[key]);
						formData[key] = utilBuildOptions[key].defaultValue;
					}
					console.log(key + " not found in form data"); 
					console.log(key + ": " + utilBuildOptions[key].helpText);
				}
				// some tidying up
				if(!formData.optimize || formData.optimize.indexOf("shrinksafe") == -1) {
					delete formData.shrinksafeServiceUrl; 
				}
				return formData; 
				//console.log("formData: " + dojo.toJson(formData, true));
				// console.dump(formData);
			} else {
				console.warn("no form object provided to get config from");
				return {};
			}
		},
		_createDialog: function() {
				var dialog = new toolbox.MessageDialog({ id: "builderDialog" });
				dojo.body().appendChild(dialog.domNode);
				dialog.startup();
				return dialog;
		},
		showDialog: function(msg, opts) {
			var dialog = dijit.byId("builderDialog") || this._createDialog();
			var args = [dialog.containerNode, msg];

			if(opts && opts.className) {
				var cls = opts.className;
				dojo.addClass(dialog.domNode, cls);
				var handle; 
				handle = dojo.connect(dialog, "hide", dojo.hitch(dialog, function() {
					dojo.removeClass(this.domNode, cls);
					dojo.disconnect(handle);
				}));
			}
			if(opts && opts.title) {
				dialog.titleNode.innerHTML = opts.title;
			}
			dialog.show();
			if(msg) {
				dialog.setContent(msg, opts || {});
			}
		},
		hideDialog: function(msg) {
			var dialog = dijit.byId("builderDialog");
			if(dialog) {
				dialog.hide();
			}
		},
		getFieldValidator: function(id) {
			var isDirectory = function(file) {
				if(file && typeof file == "string") {
					file = new air.File(file);
				}
				return Boolean(file && file.exists && file.isDirectory); 
			}; 
			var isFile = function(file) {
				if(file && typeof file == "string") {
					try {
						file = new air.File(file);
					} catch(e) {
						return false;
					}
				}
				return Boolean(file && file.exists && (!file.isDirectory)); 
			};
			var isntEmpty = function(value) {
				return Boolean(true && value.length);
			};
			var validator=function(){return true;};
			switch(id) {
				case "dojodir":
					validator = function(value) {
						var file = (value) ? new air.File(value) : {};
						var isValid = file.exists && 
							file.isDirectory && 
							file.resolvePath("dojo").exists && 
							file.resolvePath("util").exists;
						return isValid;
					};
					break;
				case "profileFile":
					validator = isFile;
					break;
				case "releaseDir":
					validator = isntEmpty;
					break;
				case "releaseName":
					validator = isntEmpty;
					break;
				default: break;
			}
			return validator;
		},
		validateBuildConfig: function(config) {
			var result = true;
			for(var key in config) {
				if(!this.getFieldValidator(key).call(this, config[key])) {
					console.warn("validateBuildConfig, bad config value: " + key + ":" + config[key] );
					return false; 
				} 
			}
			return result;
		},
		configToArgsArray: function(config) {
			var empty = {};
			var pairs = [];
			// TODO: any escaping, quoting? 
			for(var i in config) {
				// make sure the config option has a value
				if(!(i in empty) && config[i] !== "") {
					pairs.push(i + "=" + config[i]);
				}
			}
			return pairs;
		},
		configToCmdline: function(config) {
			var pairs = this.configToArgsArray(config);
			return pairs.join(" ");
		},
		runbuild: function(config){
			air.trace("runbuild");
			/* 
				summary: 
					run the dojo build using the provided settings
				description: 
					takes the baseDir value, 
					copies into that directory the dojo/air-builder files necessary to run the build in this environment
					an iframe is prepared which loads airbuild.html
					and has a sandboxbridge allow it access to some method/properties of this parent window
					loading airbuild.html loads buildrunner.js, and calls buildrunner.go() method, with the selected profile
			*/
			if(! this.validateBuildConfig(config)) {
				console.warn("build aborted: bad config");
				this.showDialog("Build configuration is incomplete", {className: "builderDialogError"});
				return;
			}
			// persist values in our comboboxes
			
			this._currentConfig = config; 
			
			console.log("fileUtil.baseDir:" + config.dojodir + "(" + typeof(config.dojodir) + ")");
			fileUtil.baseDir = (typeof config.dojodir == "string") ? new air.File(config.dojodir) : config.dojodir; 

			if(!fileUtil.baseDir.exists){
				fileUtil.baseDir.createDirectory();
			}
			air.trace("baseDir is " + fileUtil.baseDir.nativePath);
			var cwDir = fileUtil.baseDir.resolvePath(fileUtil.basePath);
			
			// TODO: if this is a repeat build we dont need to do all/some of this? 
			
			// Start by copying the util/buildscripts from the user's
			// Dojo to an airbuild directory there
			var buildScriptsDir = this.moduleDirectory.resolvePath("../../util/buildscripts");
			air.trace("copying directory " + buildScriptsDir.nativePath + " to: " + cwDir.nativePath);
			fileUtil.copyDir( buildScriptsDir.nativePath, cwDir.nativePath, /./, true); // /./ says match anything

			// Next, copy the AIR-specific code in from our app
			var appbuild = this.moduleDirectory.resolvePath("./dojo");
			air.trace("copying directory " + appbuild.nativePath + " to: " + cwDir.nativePath);
			fileUtil.copyDir(appbuild, cwDir.nativePath, /./, false);
			
			// Make the build frame
			var html_launcher = this.moduleDirectory.resolvePath("./airbuild.html");
			air.trace("copying file " + html_launcher.nativePath + " to: " + cwDir.resolvePath("airbuild.html").nativePath);
			fileUtil.copyFile(html_launcher, cwDir.resolvePath("airbuild.html").nativePath);

			// open a new window for our sandbox
			// make it relative to the base dir
			var baseUrl = fileUtil.baseDir.url
			var uri = fileUtil.getFileObj("airbuild.html").url.substring(baseUrl.length);

			//var documentRoot = "app-resource:/";
      var sandboxRoot="http://localhost/"
      var documentRoot=baseUrl; 

			//air.trace("airbuild.html will load from " + uri + ", with sandboxRoot: " + sandboxRoot + ", and documentRoot: " + documentRoot);
			//logger.info("Opening up build in " + uri);
			
			// create the iframe, with an event handler for ondominitialize
			// which will resume the build setup
			// see: http://livedocs.adobe.com/air/1/devappshtml/AboutHTMLEnvironment_5.html
			var buildplace = document.getElementById("buildplace");
			var text='<iframe id="buildframe" '
							+'width="400" height="400" ' 
							+'src="' + uri +'" sandboxRoot="'+sandboxRoot+'" documentRoot="' + documentRoot + '" '
							+'></iframe>';
				
			buildplace.innerHTML = text;
			//air.trace("Wrote iframe to buildplace: " + document.getElementById("buildframe"));

			// ondominitialize is an <iframe> attribute, not a window event
			// it will fire before the dom is created and before scripts are interpreted
			// and gives us a hook to ready the environment for any scripts being loaded into that frame
			var buildframe = document.getElementById("buildframe"); 
			buildframe.ondominitialize = dojo.hitch(this, this._setupBuildBridge);
		},
		_getChildBridge: function() {
			var buildframe = document.getElementById("buildframe"); 
			if(buildframe && buildframe.contentWindow && buildframe.contentWindow.childSandboxBridge) {
				return buildframe.contentWindow.childSandboxBridge; 
			} else {
				return null;
			}
		},
		_buildReset: function() {
			// the iframe will be reloaded, so we need to reset everything
			this._buildBridges_DONE = false; 
			var buildframe = document.getElementById("buildframe"); 
			if(buildframe.contentWindow) {
				delete buildframe.contentWindow.parentSandboxBridge;
				delete buildframe.contentWindow.childSandboxBridge;
			}
			if(this._logholder) {
				this._logholder.innerHTML = "";
			}
		},
		_setupBuildBridge: function() {
			// summary: 
			// 	establishes the parentSandboxBridge on the buildframe contentWindow
			air.trace("_setupBuildBridge");
			var buildframe = document.getElementById("buildframe"); 
			
			if(this._buildBridges_DONE) {
				throw new Error("_setupBuildBridge has already been called");
				return;
			}
			this._buildBridges_DONE = true;
			// this.showActivity();

			// setup the parentSandboxBridge for the buildframe
			var bridge = {
				// profile: profile,
				// action: action,
				print: print,
				dump: function(obj) {
					air.trace(dojo.toJson(obj, 1));
				},
				trace: function(msg) {
					air.trace("Child: " + msg);
				},
				//showActivity: dojo.hitch(this, this.showActivity),
				showActivity: dojo.hitch(this, function(title, options) {
					// air.trace("moduleBuilder: bridge.showActivity got title: " + title);
					this.showActivity(title, options);
				}),
				hideActivity: dojo.hitch(this, this.hideActivity), 
				buildDone: dojo.hitch(this, this._buildDone), 
				buildAborted: dojo.hitch(this, this._buildAborted), 
				// FIXME: reset state so a build can be re-run
				onBuildFrameLoad: dojo.hitch(this, this._buildKickoff),
				onBuildFrameUnload: dojo.hitch(this, this._buildReset),

				// networkOptimizeJs: dojo.hitch(this, this.networkOptimizeJs),

				_delegate: dojo.hitch(this, function(methodName, params) {
					var childBridge = this._getChildBridge();
					//air.trace("_delegate, calling " + methodName + " which exists? " + typeof this[methodName]);
					// could filter on what methods we can call in this way

					if(params.async) {
						var buildframe = document.getElementById("buildframe"); 
						
						var getObject = function(str, scope) {
							var parts = str.split(".");
							var obj = scope;
							while(parts.length) {
								var key = parts.shift();
								obj = scope[key]
								scope = obj;
							}
							return obj;
						};
						
						// async functions are expected to return deferreds
						var deferred = this[methodName].apply(this, params.params);
						if(!deferred ||  !(deferred instanceof dojo.Deferred)) {
							// fell at the gate and didnt return a deferred
							air.trace("call to: " + methodName + " didnt return a deferred");
							var err = deferred || {};
							childBridge[params.onFailure].call(buildframe.contentWindow, dojo.mixin({
								name: deferred.name || "Build Error", 
								message: deferred.message || "Error calling " + methodName
							}, deferred));
						} else {
							if(params.onSuccess) {
								//air.trace("adding success handler for " + methodName);
								deferred.addCallback(function(args){
									var data = args.data; 
									var requestArgs = args.request; 
									air.trace("request for: " + requestArgs.fileName + " got " + data.length + " in response");
									// send the result back over to the named function on the bridge
									//air.trace("calling " + methodName + " on the bridge, it exists? " + typeof childBridge[params.onSuccess]);
									childBridge[params.onSuccess](data, requestArgs); 
								});
							} else {
								air.trace("WARN: no success handler for " + methodName);
							}
							if(params.onFailure) {
								deferred.addErrback(function(err, requestArgs) {
									getObject(params.onFailure, childBridge)(err, requestArgs);
								});
							}
						}
					} else {
						var result = this[methodName].apply(this, params.params);
						if(result instanceof Error) {
							childBridge[params.onFailure]({
								name: result.name, 
								message: result.message
							});
						} else {
							childBridge[params.onFailure](result);
						}
					}
				})
			};

			for(var item in fileUtil){
				bridge[item] = fileUtil[item];
			}
			bridge.trace = air.trace;
			
			// place the bridge on the iframe window
			var buildframe = document.getElementById("buildframe");
			buildframe.contentWindow.parentSandboxBridge = bridge;

		},
		_startLoggingBuild: function(){
			var logholder = this._logholder = this.logWindow.document.getElementById("loglist");
			var func, msg; 
			while(this._logLines.length) {
				var newitem = document.createElement("li");
				newitem.innerHTML = this._logLines.shift(); 
				logholder.appendChild(newitem);
			}
		},
		_buildKickoff: function(config) {
			// at this point the child frame should be fully loaded, 
			// and have its own sandbox bridge set up
			air.trace("_buildKickoff");

			var buildframe = document.getElementById("buildframe");
			var bridge = null;
			try {
				true == buildframe.contentWindow.childSandboxBridge.isReady(); 
				bridge = buildframe.contentWindow.parentSandboxBridge; 
				air.trace("_buildKickoff: buildframe childSandboxBridge is ready");
			} catch(e) {
				air.trace("_buildKickoff: buildframe childSandboxBridge not ready\n" + e.message);
				return;
			}

			if(!config) {
				config = this._currentConfig;
			}
			// the builder can be run with command line input 
			// so we pass the argument pairs rather than the config hash
			var argumentPairs = (window.commandLine) ? commandLine : this.configToArgsArray(config);

			// open the log window, ready for any log output from it
			var logPopup = this.logPopup;
			if(this.logPopup) {
				this._startLoggingBuild();
			} else {
				this.logPopup = logPopup = this._createLogWindow(
						dojo.hitch(this, this._startLoggingBuild)); 
			}
			air.trace("_buildKickoff: calling startBuild with: " + argumentPairs.join(", "));
			buildframe.contentWindow.childSandboxBridge.startBuild(argumentPairs);
		},
		
		_buildDone: function(evt) {
			this.hideActivity(); 
			var config = this._currentConfig; 
			var description = evt.message; 
			var title = evt.name; 
			this.showDialog(description, {
				className: "builderDialogSuccess", 
				title: title
			});
			// TODO: other tear-down?
		},
		_buildAborted: function(evt) {
			air.trace("builderModule._buildAborted: " + dojo.toJson(evt || {}));
			if(!evt) {
				evt = {name:"Unknown Error", message: ""};
			}
			this.hideActivity(); 
			var description = evt.message; 
			var title = evt.name || "Build Error"; 
			this.showDialog(description, {
				className: "builderDialogFailure", 
				title: title
			});
			// TODO: other tear-down?
		},
		networkOptimizeJs: function(fileParams) {
			// returns a deferred
			air.trace("moduleBuilder: networkOptimizeJs " + fileParams.fileName + " for shrinksafe");

			if(fileParams.optimizeType.indexOf("shrinksafe") > -1) {
				// get and return a deferred
				return this._fetchShrinkSafedFile(fileParams);
			} else {
				return new Error("networkOptimizeJs couldnt optimize unknown optimizeType: " + fileParams.optimizeType); 
			}
		},
		
		_fetchShrinkSafedFile: function(requestArgs) {
			// Can we safely make assumptions about the baseDirectory
			// Or do we always need to pass through the full pathname (.nativePath)
			// (which might contain semi-sensitive info like customer/project name)
			// returns dojo.Deferred
			var file = new air.File(requestArgs.fileName); 
			air.trace("_fetchShrinkSafedFile, fetching: " + file.nativePath + ", which exists? " + file.exists);
			if(!requestArgs.serviceUrl) {
				console.warn("request for "+requestArgs.fileName+" had no serviceUrl");
			}
			var deferred = new dojo.Deferred(); 
			
			var ssRequest = new toolbox.ShrinkSafeRequest({
				target: requestArgs.serviceUrl, 
				fileParamName: "shrinkfile[]",
				//baseDirectory: requestArgs.baseDirectory
			});
			// sanity check
			if(!(ssRequest.target && /^[a-z]{3,}:\/.+/.test(ssRequest.target))) {
				// bad params, return an Error
				return new Error("Bad ShrinkSafeRequest target url: " +ssRequest.target); 
			}
			// here's where we'd add any other options, keepLines is the only one so far
			ssRequest.setOption("stripnewlines", !requestArgs.keepLines);

			ssRequest.onLoad = function(strResponse) {
				air.trace("ShrinkSafeRequest.onLoad: got strResponse of length: " + strResponse.length + " for file: " + requestArgs.fileName);
				deferred.callback({data: strResponse, request: requestArgs});
			};
			ssRequest.onError = function(err) {
				console.warn("ShrinkSafeRequest.onLoad: Error from shrinksafe request: " + err.message);
				deferred.errback({name: "ShrinkSafeRequest Error", message: err.message}, requestArgs);
			};
			
			if(ssRequest.addFile(file)) {
				this.showActivity("Requesting ShrinkSafe-d file", {
					indeterminate: true 
				});
				// dispatch the request
				ssRequest.send();
			} else {
				return new Error("ShrinkSafe Error failed to addFile: " + file); 
			}
			return deferred;
		},

		selectDir : function(callback, opts) {
			params = dojo.mixin({
				prompt: "Select Directory"
			}, opts);
			var dir = new air.File();
			console.log("browsing for dir");
			try {
					dir.browseForDirectory(params.prompt);
			    dir.addEventListener(air.Event.SELECT, function(event) {
				    callback(event.target);
					});
			} catch (error) {
			    console.log("selectDir failed:", error.message);
			}
		},
		selectFile : function(callback, opts) {
			params = dojo.mixin({
				fileType: "Text",
				extensions: "*.*",
				prompt: "Select File"
			}, opts);
			var profileFile = new air.File();
			var filters = [
				new air.FileFilter(params.fileType, params.extensions)
			];
			console.log("browsing for profile");
			try {
			    profileFile.browseForOpen(params.prompt, filters);
			    profileFile.addEventListener(air.Event.SELECT, function(event) {
				    callback(event.target);
					});
			} catch (error) {
			    console.log("selectFile failed:", error.message);
			}
		}, 
		
		_createLogWindow: function(callback) {
			// createa  popup to display the output log
			console.log("creating log popup");
			// initialize the array we'll store log lines in until the logger window's dom is ready for them
			this._logLines = [];
			this._logholder = null; 
			
			// assemble the window.open params
			name = "logWindow";
			var url = this.moduleDirectory.url + "/logWindow.html";
			
			var options = new air.NativeWindowInitOptions(); 
				
			var w = 600,
				h = 300;
			// TODO: position next to the module window, not over it
			var windowBounds = new air.Rectangle(
				(air.Capabilities.screenResolutionX - w) / 2,
				air.Capabilities.screenResolutionY - h,
				w, h
			);

			var logPopup = this.logPopup = air.HTMLLoader.createRootWindow(true, options, true, windowBounds);
			this.logWindow = logPopup.window;
			logPopup.load( new air.URLRequest(url) );

			logPopup.addEventListener("complete", callback); 
			
			logPopup.addEventListener("close", dojo.hitch(this, function(e){
				delete this._logholder;
				delete this.logWindow;
				delete this.logPopup;
				logPopup.removeEventListener("complete", callback); 
			})); 
			
			logPopup.window.nativeWindow.activate();

			return logPopup;
		}
	});

	function print(message) {
		// the property is available immediately, but the dom not.
		console.log("print: " + message);

		if(builderModule._logholder) {
			var logholder = builderModule._logholder;
			var newitem = document.createElement("li");
			newitem.innerHTML = message || "boo";
			logholder.appendChild(newitem);
		} else {
			// do it async - add to the list to write out when the dom's ready
			builderModule._logLines.push(message);
		}
	};
	
})();
