trace = parentSandboxBridge.trace;

function globalEval(/*String*/jsString){
	var script = document.createElement("script");
	script.type = "text/javascript";
	script.defer = false;
	script.text = jsString;
	var headNodeSet = document.getElementsByTagName("head");
	if(headNodeSet.length){
		headNodeSet.item(0).appendChild(script);
	}
}

function dojoLoad(/*String*/ filename, /*Function*/ cb){
	/* Stands in for the dojo._loadUri function */
	contents = fileUtil.readFile(filename);
	if(cb){
		contents = '('+contents+')';
		var value = dojo.eval(contents);
		cb(value);
	} else {
		dojo.eval(contents);
	}
	return true;
}

function load(filename) {
	/* Reads and evaluates the file. Also installs a special
	function into dojo for loading, if needed. */
	
	// Install our load handler 
	if(window.dojo && !dojo._dependency_loader && dojo._loadUri != dojoLoad){
		dojo._loadUri = dojoLoad;
	}
	contents = fileUtil.readFile(filename);
	globalEval(contents);
	return true;
};

// pretend to be spidermonkey for now
function line2pc(fun, line){
	return "hello";
}


var buildrunner = {
	//********* Start help ************
	help : function(){
		var buildOptionText = "";
		for(var param in buildUtil.DojoBuildOptions){
			buildOptionText += param + "=" + buildUtil.DojoBuildOptions[param].defaultValue + "\n"
				+ buildUtil.DojoBuildOptions[param].helpText + "\n\n";
		}

		var helpText = "To run the build, you must have Java 1.4.2 or later installed.\n"
			+ "To run a build run the following command from this directory:\n\n"
			+ "> java -jar ../shrinksafe/custom_rhino.jar build.js [name=value...]\n\n"
			+ "Here is an example of a typical release build:\n\n"
			+ "> java -jar ../shrinksafe/custom_rhino.jar build.js profile=base action=release\n\n"
			+ "The possible name=value build options are shown below with the defaults as their values:\n\n"
			+ buildOptionText;

		print(helpText);
	},
	//********* End help *********

	//********* Start clean ************
	clean : function(kwArgs){
		logger.info("Deleting: " + kwArgs.releaseDir);
		fileUtil.deleteFile(kwArgs.releaseDir);
	},
	//********* End clean *********
	
	// helper for exception logging that is used during the 'release'
	// action
	logException : function(section, e){
		logger.error("Error in build (" + section + "): " + e);
		if(e.stackTrace){
			logger.error("Stack trace:");
			for(var i = 0; i < e.stackTrace.length; i++){
				logger.error("  " + e.stackTrace[i].line + " - " + e.stackTrace[i].sourceURL);
			}
		}
		throw "Build failed";
	},
	
	//********* Start release *********
	release : function(kwArgs){
		buildrunner.longrunning = true;
		trace("in release");
		// to avoid having to patch too much of buildUtils, we stash the serviceUrl for use in optimizeJs
		buildrunner.serviceUrl = kwArgs.shrinksafeServiceUrl;
		logger.info("Using profile: " + kwArgs.profileFile);
		logger.info("Using version number: " + kwArgs.version + " for the release.");
		
		buildrunner.showActivity("Setting up", {indeterminate: false, progress: 0});
		
		try
		{
			var dependencies = kwArgs.profileProperties.dependencies;
			var prefixes = dependencies.prefixes;
			var lineSeparator = fileUtil.getLineSeparator();
			var copyrightText = fileUtil.readFile("copyright.txt");
			var buildNoticeText = fileUtil.readFile("build_notice.txt");
			//Find the dojo prefix path. Need it to process other module prefixes.
			var dojoPrefixPath = buildUtil.getDojoPrefixPath(prefixes);
		} catch(e) {
			buildrunner.logException("setup", e);
			throw e;
		}

		//Convert targeted build layers to an array.
		var buildLayers = null;
		if(kwArgs.buildLayers){
			//Make sure to copy over any "source" files for the layers be targeted by
			//buildLayers. Otherwise dependencies will not be calculated correctly.
			buildLayers = kwArgs.buildLayers.split(",");
		}

		function setup(){
			trace("release: in setup");
			if(!kwArgs.buildLayers){
				buildrunner.clean(kwArgs);
			}
			
			i = 0;
			buildrunner.showActivity("Copying files", {indeterminate: false, progress: 5});
			window.timers.copyFiles = setTimeout(copyFiles, 100);
		}
		
		window.timers.setup = setTimeout(setup, 10);
		var prefixName = null;
		
		var i = 0;
		function copyFiles()
		{
			trace("release: in copyFiles");
			//Get the list of module directories we need to process.
			//They will be in the dependencies.prefixes array.
			//Copy each prefix dir to the releases and
			//operate on that copy instead of modifying the source.
			try {
				prefixName = prefixes[i][0];
				var prefixPath = prefixes[i][1];

				var finalPrefixPath = prefixPath;
				if(finalPrefixPath.indexOf(".") == 0 && prefixName != "dojo"){
					finalPrefixPath = dojoPrefixPath + "/" + prefixPath;
				}
				buildrunner._copyToRelease(prefixName, finalPrefixPath, kwArgs, buildLayers);

				if(kwArgs.symbol){
					var releasePath = kwArgs.releaseDir + "/"  + prefixName.replace(/\./g, "/");
					buildUtil.insertSymbols(releasePath, kwArgs);
				}
			} catch(e) {
				buildrunner.logException("copying", e);
				throw e;
			}
			i++;
			if(i < prefixes.length){
				var curper = (i / prefixes.length) * 55 + 5;
				buildrunner.showActivity("Copying files", 
					{indeterminate: false, progress: curper});
				window.timers.copyFiles = setTimeout(copyFiles, 100);
			} else {
				buildrunner.showActivity("Build dojo.js and layer files", 
					{indeterminate: false, progress: 60});

				window.timers.buildDojo = setTimeout(buildDojo, 10);
			}
		}
		
		var result = null;
		//Save the build layers. The first layer is dojo.js.
		var defaultLegalText = copyrightText + buildNoticeText;
		var dojoReleaseDir = kwArgs.releaseDir + "/dojo/";
		var layerIgnoreString = "";
		var nlsIgnoreString = "";
		var optimizeType = kwArgs.optimize; 

		
		function buildDojo()
		{
			trace("release: in buildDojo");
			//Fix all the prefix paths to be in the release directory.
			//Do this after the copy step above. If it is done as part
			//of that loop, then dojo path gets set first usually, and any prefixes
			//after it are wrong.
			for(i = 0; i < prefixes.length; i++){
				prefixes[i][1] = kwArgs.releaseDir + "/"  + prefixes[i][0].replace(/\./g, "/");
			}

			//Make sure dojo is clear before trying to map dependencies.
			if(typeof dojo != "undefined"){
				dojo = {};
			}
			
			try {
				logger.trace("Building dojo.js and layer files");
				result = buildUtil.makeDojoJs(buildUtil.loadDependencyList(kwArgs.profileProperties, kwArgs), kwArgs.version, kwArgs);
			} catch(e) {
				buildrunner.logException("Creating dojo.js and layer files", e);
				throw e;
			}

			//Add an ending comma to the list to make matches easier.
			//Also make sure we normalize to unix path separators.
			if(kwArgs.buildLayers){
				kwArgs.buildLayers += ",";
				kwArgs.buildLayers = kwArgs.buildLayers.replace(/\\/g, "/");
			}
			i = 0;

			window.timers.optimizeResult = setTimeout(function() {
				// wrap in a function to close over kwArgs
				optimizeResult();
			}, 100);
		}
		
		function optimizeResult() {
			var curper = (i / result.length)*35 + 60;
			buildrunner.showActivity("Optimizing result files", 
				{indeterminate: false, progress: curper});

			//	trace("in optimizeResult, kwArgs is: ");
			//	dump(kwArgs || {});

			var currentLayer = result[i];
			var layerName = currentLayer.layerName;
			var layerLegalText = (currentLayer.copyrightFile ? fileUtil.readFile(currentLayer.copyrightFile) : defaultLegalText);
			var fileName = dojoReleaseDir + currentLayer.layerName;
			var fileContents = currentLayer.contents;

			//Build up string of files to ignore for the directory optimization step
			var ignoreName = layerName.replace(/\.\.\//g, "");
			var nameSegment = ignoreName.replace(/\.js$/, "");
			layerIgnoreString += (layerIgnoreString ? "|" : "") + buildUtil.regExpEscape(ignoreName) + "$";
			layerIgnoreString += "|" + buildUtil.regExpEscape(ignoreName + ".uncompressed.js") + "$";

			if(nameSegment.indexOf("/") != -1){
				nameSegment = nameSegment.substring(nameSegment.lastIndexOf("/") + 1, nameSegment.length);
			}
			nlsIgnoreString += (nlsIgnoreString ? "|" : "") + buildUtil.regExpEscape(nameSegment);

			//If only want to build certain layers, skip ones that do not match.
			if(kwArgs.buildLayers && kwArgs.buildLayers.indexOf(layerName + ",") == -1){
				return;
			}
			trace("optimizeResult: adding djConfig");
			//Burn in djConfig for dojo.js/xd.js if requested.
			if(kwArgs.scopeDjConfig && (layerName.match(/dojo\.xd\.js$/) || layerName.match(/dojo\.js$/))){
				fileContents = buildUtil.setScopeDjConfig(fileContents, kwArgs.scopeDjConfig);
			}

			trace("optimizeResult: adding scope names");
			//Burn in scope names for dojo.js/xd.js if requested.
			if(kwArgs.scopeMap && (layerName.match(/dojo\.xd\.js$/) || layerName.match(/dojo\.js$/))){
				fileContents = buildUtil.setScopeNames(fileContents, kwArgs.scopeMap);
			}

			trace("optimizeResult: adding xdDojoPath");
			//Burn in xd path for dojo if requested, and only do this in dojo.xd.js.
			if(layerName.match(/dojo\.xd\.js/) && kwArgs.xdDojoPath){
				fileContents = buildUtilXd.setXdDojoConfig(fileContents, kwArgs.xdDojoPath);
			}

			trace("optimizeResult: Flatten resources");
			//Flatten resources
			fileContents = i18nUtil.flattenLayerFileBundles(fileName, fileContents, kwArgs);

			//Save uncompressed file.
			var uncompressedFileName = fileName + ".uncompressed.js";
			var uncompressedContents = layerLegalText + fileContents;
			if(layerName.match(/\.xd\.js$/) && !layerName.match(/dojo(\.xd)?\.js/)){
				uncompressedContents = buildUtilXd.makeXdContents(uncompressedContents, prefixes, kwArgs);
			}
			fileUtil.saveUtf8File(uncompressedFileName, uncompressedContents);

			//Intern strings if desired. Do this before compression, since, in the xd case,
			//"dojo" gets converted to a shortened name.
			if(kwArgs.internStrings){
				logger.info("Interning strings for file: " + fileName);
				var prefixes = dependencies["prefixes"] || [];
				var skiplist = dependencies["internSkipList"] || [];
				buildUtil.internTemplateStringsInFile(uncompressedFileName, dojoReleaseDir, prefixes, skiplist);

				//Load the file contents after string interning, to pick up interned strings.
				fileContents = fileUtil.readFile(uncompressedFileName);
			}else{
				fileContents = uncompressedContents;
			}

			//Save compressed layer file.
			var compressedContents = buildUtil.optimizeJs(fileName, fileContents, layerLegalText, kwArgs.optimize);
			if(compressedContents) {
				fileUtil.saveUtf8File(fileName, compressedContents);
			}

			i++;
			if(i < result.length){
				window.timers.optimizeResult = setTimeout(function() {
					// close over to keep kwArgs in scope
					optimizeResult();
				}, 50);
			} else {
				buildrunner.showActivity("Final build steps", 
					{indeterminate: true, progress: "0", places: 0, maximum: 100});
				window.timers.optimizeResult = setTimeout(function() {
					finalSteps();
				}, 100);
			}
		}
		
		function finalSteps(){
			//Save the dependency lists to build.txt
			trace("finalSteps");
			var buildText = "Files baked into this build:" + lineSeparator;
			for(var i = 0; i < result.length; i++){
				buildText += lineSeparator + result[i].layerName + ":" + lineSeparator;
				buildText += result[i].depList.join(lineSeparator) + lineSeparator;
			}
			fileUtil.saveFile(kwArgs.releaseDir + "/dojo/build.txt", buildText);
			logger.info(buildText);

			//Run string interning, xd file building, etc.. on the prefix dirs in the
			//release area.
			var layerIgnoreRegExp = new RegExp("(" + layerIgnoreString + ")");
			var nlsIgnoreRegExp = new RegExp("\\/nls\\/(" + nlsIgnoreString + ")_");

			for(var i = 0; i < prefixes.length; i++){
				var copyrightText = null;
				if(prefixes[i][2]){
					copyrightText = fileUtil.readFile(prefixes[i][2]);
				}

				//Optimize the release dirs, but only if we are not building just a layer.
				if(!kwArgs.buildLayers){
					trace("calling _optimizeReleaseDirs as buildLayers was: " + kwArgs.buildLayers);
					buildrunner._optimizeReleaseDirs(prefixes[i][0], prefixes[i][1], copyrightText, kwArgs, layerIgnoreRegExp, nlsIgnoreRegExp);
				} else {
					trace("skipping _optimizeReleaseDirs as buildLayers was: " + kwArgs.buildLayers);
				}
			}

			//Copy over DOH if tests where copied.
			if(kwArgs.copyTests){
				trace("copying doh as kwArgs.copyTests is: " + kwArgs.copyTests);
				// copyRegExp = new RegExp(prefixName.replace(/\\/g, "/") + "/(?!tests)");

				fileUtil.copyDir("../../util/doh", kwArgs.releaseDir + "/util/doh", '.'); 
			} else {
					trace("skipping copying doh as kwArgs.copyTests is: " + kwArgs.copyTests);
			}
			logger.info("Build is in directory: " + kwArgs.releaseDir);
			buildrunner.longrunning = false;
		}
	},
	//********* End release *********

	//********* _remoteshrinksafe ************
	// this is a system-added action that runs after release
	// if optimize=remote has been specified
	// making it a seperate action gives us control over progress indication
	// and makes it run entirely after the initial release action
	_remoteshrinksafe : function(kwArgs){
		trace("_remoteshrinksafe action is started");
		buildrunner.longrunning = true;
		var fileobjects = buildrunner._filesForRemoteOptimization || [];
		buildrunner._remoteshrinksafeFileCount = fileobjects.length; 
		
 		logger.info("_remoteshrinksafe action, " + fileobjects.length + " to compress");
		buildrunner._remoteshrinksafeNext();
	},
	
	_remoteshrinksafeNext : function() {
		// send the next file over for optimization
		var fileobjects = buildrunner._filesForRemoteOptimization;
		var fileCount = fileobjects.length; 
		trace("_remoteshrinksafeNext, fileCount: " + fileCount);
		
		if(fileCount) {
			var params = fileobjects.shift();
			var index = buildrunner._remoteshrinksafeFileCount - fileCount; 
			logger.info("Requesting remote shrinksafe of: " + params.fileName + " with serviceUrl: " + params.serviceUrl);

			//trace("calling buildrunner.showActivity");
			buildrunner.showActivity(params.fileName, {
				title: "Remote Shrinksafe", 
				indeterminate: false, 
				progress: index,
				maximum: buildrunner._remoteshrinksafeFileCount 
			});
			// request the optimized file from the indicated service
			// we cant pass a callback function ref over the sandbox bridge, 
			// it knows to call jsOptimizeHandler (on the child bridge), which calls remoteshrinksafeCallback with the result
			try{
				//trace("calling delegateToParent with networkOptimizeJs");
				delegateToParent( "networkOptimizeJs", params, {
					async: true,
					// callbacks are **string** names
					onSuccess: "shrinkSafeResponseHandler", 
					onFailure: "shrinkSafeErrorHandler"
				});
			} catch(e) {
				trace("_remoteshrinksafeNext: Caught error calling networkOptimizeJs (357) - " + e.message);
				buildAborted({name: "Shrinksafe build error", message: e.message});
			}
		} else {
			// done!
			trace("no more fileobjects, were done");
			buildrunner.longrunning = false;
		}
		
	},
	//********* End _remoteshrinksafeNext *********
	
	_remoteshrinksafeCallback : function(data, requestParams) {
		// we're expecting a string, 
		// if data duck-types to an Error (it gets serialized so it wont instanceof Error)
		if(typeof data == "object" && data.message) {
			// urgg. abort.
			// 
			var err = data; 
			alert("Shrinksafe failure: " + err.message + "\nAborting.");
			buildAborted(err);
		} else {
			trace("got back data for: " + requestParams.fileName);
			fileUtil.saveUtf8File(requestParams.fileName, requestParams.prepend + data + requestParams.postpend);
			trace("wrote out compressed file: " + requestParams.fileName);
			buildrunner._remoteshrinksafeNext();
		}
	},
	
	//********* _scheduleFileForRemoteOptimization ************
	_scheduleFileForRemoteOptimization : function(/*String fileName*/fileName, /*Object*/params) {
		// Just add the details to an array for later processing
		if(!buildrunner._filesForRemoteOptimization) {
			buildrunner._filesForRemoteOptimization = []; 
			buildrunner._fileKeysForRemoteOptimization = {}; 
		}
		if(fileName in buildrunner._fileKeysForRemoteOptimization) {
			trace("WARN: " + fileName + " is already scheduled for shrinksafing");
		}
		trace("adding fileName ForRemoteOptimization: " + fileName)
		buildrunner._filesForRemoteOptimization.push(params);
		buildrunner._fileKeysForRemoteOptimization[fileName] = params;
	},
	//********* End _scheduleFileForRemoteOptimization *********
	
	
	//*****************************************************************************
	//********* Start buildrunner._copyToRelease *********
	_copyToRelease : function (/*String*/prefixName, /*String*/prefixPath, /*Object*/kwArgs, /*Array?*/buildLayers){
		//summary: copies modules and supporting files from the prefix path to the release
		//directory. Also adds code guards to module resources.
		var prefixSlashName = prefixName.replace(/\./g, "/");
		var releasePath = kwArgs.releaseDir + "/"  + prefixSlashName;
		var copyRegExp = /./;

		//Use the copyRegExp to filter out tests if requested.
		if(!kwArgs.copyTests){
			copyRegExp = new RegExp(prefixName.replace(/\\/g, "/") + "/(?!tests)");
		}

		logger.info("Copying: " + prefixPath + " to: " + releasePath);
		var copiedFiles = fileUtil.copyDir(prefixPath, releasePath, copyRegExp.source, true);

		//Make sure to copy over any "source" files for the layers be targeted by
		//buildLayers. Otherwise dependencies will not be calculated correctly.
		if(buildLayers){
			for(i = 0; i < buildLayers.length; i++){		
				var relativeLayerPath = buildLayers[i].replace(/\.\.\//g, "");

				//See if relativeLayerPath has teh prefix slash name in it.
				//This means the layer is probably in this prefix dir (but no guarantee)
				//This is a bit hacky.
				if(relativeLayerPath.indexOf(prefixSlashName) == 0){

					//Remove the prefix part from the dir and add the prefix path to get a
					//full path.
					var layerPathSuffix = relativeLayerPath.replace(prefixSlashName, "");
					relativeLayerPath = prefixPath + layerPathSuffix;

					//If that source path exists, it means we need to copy over the source
					//layer file.
					var cwDir = fileUtil.baseDir.resolvePath(fileUtil.basePath);
					if( fileUtil.fileExists(cwDir.resolvePath(relativeLayerPath)) ){
						//Need to copy over from the source area.
						var destPath = releasePath + layerPathSuffix;
						fileUtil.copyFile(relativeLayerPath, destPath);
					}
				}
			}
		}

		//Put in code guards for each resource, to protect against redefinition of
		//code in the layered build cases. Do this here before the layers are built.
		if(copiedFiles){
			buildUtil.addGuards(copiedFiles);
		}
	},
	//********* End buildrunner._copyToRelease *********

	//********* Start buildrunner._optimizeReleaseDirs *********
	_optimizeReleaseDirs : function (
		/*String*/prefixName, 
		/*String*/prefixPath,
		/*String*/copyrightText,
		/*Object*/kwArgs,
		/*RegExp*/layerIgnoreRegExp,
		/*RegExp*/nlsIgnoreRegExp){	
		//summary: runs intern strings, i18n bundle flattening and xdomain file generation
		//on the files in a release directory, if those options are enabled.
		var releasePath = kwArgs.releaseDir + "/"  + prefixName.replace(/\./g, "/");
		var prefixes = kwArgs.profileProperties.dependencies.prefixes;

		//Intern strings if desired.
		if(kwArgs.internStrings){
			logger.info("Interning strings for: " + releasePath);
			buildUtil.internTemplateStrings(kwArgs.profileProperties.dependencies, releasePath, layerIgnoreRegExp);
		}

		//Process build conditionals in non-layer module files.
		buildUtil.processConditionalsForDir(releasePath, layerIgnoreRegExp, kwArgs);
		
		//Flatten bundles inside the directory
		i18nUtil.flattenDirBundles(prefixName, prefixPath, kwArgs, nlsIgnoreRegExp);	
		
		if(kwArgs.loader == "xdomain"){
			buildUtilXd.xdgen(prefixName, prefixPath, prefixes, layerIgnoreRegExp, kwArgs);
		}
		
		//FIXME: call stripComments. Maybe rename, inline with optimize? need build options too.
		if(kwArgs.optimize){
			buildUtil.stripComments(releasePath, layerIgnoreRegExp, copyrightText, kwArgs.optimize);
		}
		
		if(kwArgs.cssOptimize){
			buildUtil.optimizeCss(releasePath, kwArgs.cssOptimize, kwArgs.cssImportIgnore);
		}
	}
	//********* End buildrunner._optimizeReleaseDirs *********
	
};

function monkeypatch_buildutil(kwArgs){
	buildUtil.optimizeJs = function(/*String fileName*/fileName, /*String*/fileContents, /*String*/copyright, /*String*/optimizeType, /*String?*/serviceUrl){
		//summary: either strips comments from string or compresses it.
		trace("buildUtil.optimizeJs: " + fileName + "("+optimizeType+")");
		serviceUrl = serviceUrl || buildrunner.serviceUrl;
		copyright = copyright || "";

		//Save or schedule-for-save the compressed file.
		var returnValue = null; 
		
		switch(optimizeType) {
			case "shrinksafe.keepLines": 
				// just register the file for later when remoteshrinksafe action runs
				// itll need to prepend the copyright, and fileUtil.saveUtf8File(fileName, compressedContents);
				buildrunner._scheduleFileForRemoteOptimization(fileName, {
					'fileName': fileName, 
					'fileContents': fileContents, 
					'prepend': copyright, 
					'postpend': "", 
					'optimizeType': optimizeType, 
					'keepLines': true, 
					'serviceUrl': serviceUrl
				});
				returnValue = fileContents; 
				break; 
			case "shrinksafe": 
				// just register the file for later when remoteshrinksafe action runs
				// itll need to prepend the copyright, and fileUtil.saveUtf8File(fileName, compressedContents);
				buildrunner._scheduleFileForRemoteOptimization(fileName, {
					'fileName': fileName, 
					'fileContents': fileContents, 
					'prepend': copyright, 
					'postpend': "", 
					'optimizeType': optimizeType, 
					'keepLines': false,
					'serviceUrl': serviceUrl
				});
				returnValue = fileContents; 
				break; 
			case "packer":
			// not yet supported, fall thru
			case "comments":
			// already done, fall thru
			default: 
				returnValue = copyright + fileContents; 
				break;
		}

		return returnValue;
	}
};

// Flag that represents a long running action that is still going.
// This is structured so that long running tasks can return immediately
// without holding up the UI (for progress meters)
buildrunner.longrunning = false;

buildrunner.go = function() {
	//Main build script for Dojo
	//for progress indication, there's some distinct phases to the build
	// startup - munging input, prep-ing the environment: 3%
	// clean build action 10%
	// release build action 90%
	
	var buildTimerStart = (new Date()).getTime();
	trace("buildrunner.go");
	logger.level = logger.TRACE; 
	logger.trace("buildrunner.go");

	// AIRXXX don't use custom rhino stuff in AIR
	monkeypatch_buildutil();
	
	//NOTE: See buildUtil.DojoBuildOptions for the list of build options.

	//*****************************************************************************
	//Convert arguments to keyword arguments of the form key=value key2=value, 
	// like you'd get off the cmdline
	var arArgs = [];
	// convert arguments to a real array
	for(var i=0; i<arguments.length; i++) {
		arArgs.push(arguments[i]);
	}
	logger.trace("buildUtil.makeBuildOptions");
	var kwArgs = buildUtil.makeBuildOptions(arArgs);
	
	//Set logging level.
	logger.level = kwArgs["log"];
	
	var action = kwArgs.action;
	
	// add an extra _remoteshrinksafe action to do js optimization at the end
	// FIXME: there's seperate settings for module files (optimize) and layer files (layerOptimize)
	if(kwArgs.optimize.indexOf("shrinksafe") > -1) {
		logger.trace("adding _remoteshrinksafe action");
		action.push("_remoteshrinksafe");
	}
	var actionIndex = 0;
	
	function waitForAction(){
		// for long running tasks, we'll wait a bit before
		// checking again to see if they're done.
		if(buildrunner.longrunning){
			window.timers.waitForAction = setTimeout(waitForAction, 250);
		} else {
			window.timers.runAction = setTimeout(runAction, 1);
		}
		
	}
	function runAction() {
		trace("runAction");
		if(actionIndex < action.length){
			trace("runAction: " + action[actionIndex]);
			logger.logPrefix = action[actionIndex] + ": ";
			buildrunner[action[actionIndex]](kwArgs);
			actionIndex++;
			if(buildrunner.longrunning){
				window.timers.waitForAction = setTimeout(waitForAction, 250);
				return;
			}
		}
		logger.logPrefix = "";
		if(actionIndex < action.length){
			trace("runAction: " + actionIndex + "/" + action.length + " theres more, " + action[actionIndex] + " is next");
			window.timers.runAction = setTimeout(runAction, 1);
		}else{
			var buildTime = ((new Date().getTime() - buildTimerStart) / 1000);
			logger.info("Build time: " + buildTime + " seconds");
			trace("runAction: that was the last action");
			buildDone({ 
				name: "Build complete", 
				message: "Your build is located in: "+ fileUtil.getAbsolutePath(kwArgs.releaseDir)
			}); 
		}
	}
	window.timers.runAction = setTimeout(runAction, 1);
};

buildrunner.global = this;
