dojo.provide("builder.widget.GrowingTable");

dojo.require("dojo.html");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.declare("builder.widget.GrowingTable", [dijit._Widget], {
	// a widget that knows how to duplicate its srcNodeRef node 
	// and increment ids etc. appropriately
	lastIndex: 0, 
	parseWidgets: false, 
	
	buildRendering: function() {
		// the first content row is the template row
		this._contentRows=this.srcNodeRef.tBodies[0].rows;
		var tmplNode = this._contentRows[0];

		this._cleanTemplateNode( tmplNode ); 

		this._tmpTbody = dojo.doc.createElement("tbody");
		
		this.rowTemplateString = this._getRowTemplateString(tmplNode);

		this.domNode = this.srcNodeRef;
	},
	
	postCreate: function() {
		var rowNode; 
		
		for(var idx=this._contentRows.length-1, lastIndex=this.lastIndex; idx<lastIndex; idx++) {
			this.appendRow();
		}
	},
	onAppendRow: function(rowNode) {
		// hook to allow post processing
		// console.log("appended: " + rowNode);
	},
	onRemoveRow: function(rowNode) {
		// hook to allow post processing
	},
	appendRow:function() {
		var rowNode = this._createIncrementedClone()
		this.domNode.tBodies[0].appendChild(rowNode);
		this.onAppendRow(rowNode);
	},
	removeRow:function(row) {
		// console.log("removingRow: "+ row);
		var rowNode;
		if(!row) {
			rowNode = this._contentRows[this._contentRows.length-1];
		} else if(row.nodeType && row.nodeType==1 && row.tagName=="TR") {
			rowNode = row;
		} else if(Number(row)) {
			rowNode = this._contentRows[row];
		}
		if(!rowNode) {
			return;
		}
		this.domNode.tBodies[0].removeChild(rowNode);
		this.onRemoveRow(rowNode);
		if(!this._contentRows.length) {
			this.appendRow();
		}
	},
	_cleanTemplateNode: function(node) {
		dojo.forEach(["startIndex", "endIndex", "dojoType"], function(attr) {
			dojo.removeAttr(node, attr);
		})
		return node; 
	},
	_getRowTemplateString: function(node){
		if(node.parentNode) {
			node = node.cloneNode(true);
		}

		dojo.html.set(this._tmpTbody, node);
		// console.log("_getRowTemplateString got: " + node);

		var html = this._tmpTbody.innerHTML;
		this._tmpTbody.innerHTML = "";
		return html;
	},
	_createIncrementedClone: function(lastIndex) {
		// doing this string-wise - could also be done DOM-wise
		// webkit gives us outerHTML, this would be a little harder otherwise
		var rowString = this.rowTemplateString;
		var count = lastIndex || ++this.count;

		// console.log(rowString);
		var re = /(id|name)=("|')([a-zA-Z_\-]+)("|')/g; 
		// console.log("matched?" + re.test(rowString));
		// console.log("_createIncrementedClone with: "+ count);
		rowString = rowString.replace(
			re, 
			function(matchStr, attr, quot, value, endquot, offset, str) { 
				var subs = [
					attr,
					"=",
					quot,
					// TODO: match & increment an existing suffix
					value+"_"+count,
					endquot
				].join("");
				// console.log("matched: " + matchStr + "\nreturning: " + subs);
				
			}
		);
		dojo.html.set(this._tmpTbody, rowString); 
		var rowNode = this._tmpTbody.removeChild(this._tmpTbody.firstChild);
		
		console.log("_createIncrementedClone made: "+ rowNode);
		if(this.parseWidgets) {
			var newWidgets = dojo.parser.parse(rowNode); 
			// TODO: do what with the child widgets? 
		}
		this.lastIndex = count;
		return rowNode;
	}
});
