dojo.provide("profileEditor.core");

dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit._Templated");
dojo.require("dijit.Tree");
dojo.require("dijit.form.Button");
dojo.require("toolbox.ActivityOverlay");
dojo.require("toolbox.Module");
dojo.require("dojox.data.jsonPathStore");

dojo.registerModulePath("profileTemplates", "app:/modules/profileEditor/resources/");

// the json object for our Tree
profileEditor._cache = {};
profileEditor._weKnowBase = false;
profileEditor._basePath = null;

dojo.declare("profileEditor.Container", dijit.layout.TabContainer, {
	
	loadProfile: function(/* Obj|String */profile){
		// load a profile (obj), and create tabs from layers, etc
		dojo.forEach(this.getChildren(),function(child){
			air.trace(child.id);
		});
	},
	
	addChild: function(child){
		// summary: same as normal addChild, but this one adds a setTitle
		// method to the ContentPane if it doesn't have one
		this.inherited(arguments);
		if(!child["setTitle"] && !child["setDisabled"] && child.title){
			dojo.mixin(child,{ 
				setTitle: function(title){
					// summary: set the title 
					this.title = title;
					this.controlButton.containerNode.innerHTML = title || ""; 
				}
			});
		}
	}
	
});

dojo.declare("profileEditor.Profile", [dijit.layout.ContentPane, dijit._Templated], {

	widgetsInTemplate:true,

	templatePath: dojo.moduleUrl("profileTemplates", "_Profile.html"),
	
	constructor: function(){
		this.prefixes = {};
	},
	
	postCreate: function(){
		if(this.profileName){
			this.profileName.value = this.title;
		}
		this._setTitle();
		this._scanPrefixes();
		this.inherited(arguments);
	},
	
	save: function(){
		// summary: validate and save this profile, make clean again
		this.isDirty = false;
		this._setTitle.apply(this, arguments);
		// now save, 
	},
		
	_setTitle: function(args){
		if(this["setTitle"]){
			this.setTitle.call(this, this.title);
		}
	},
	
	_handleChange: function(e){
		// summary: handle changes in the options area
	},
	
	_scanPrefixes: function(){
		if(this.basePath){
			dojo.forEach(this.basePath.getDirectoryListing(), function(file){
				if(file.isDirectory && !file.name.match(/^\./)){
					// add new table row for this prefix, might sniff *.js in this folder to be sure
					if(file.name !== "util"){
						this.prefixes[file.name] = "../" + file.name;
					}
				}
			},this);
			
			for(var i in this.prefixes){
				var tr = dojo.doc.createElement('tr');
				tr.innerHTML = "<td><input type='checkbox' checked='checked'></td><td>" + i + "</td><td>" + this.prefixes[i] + "</td>";
				this.prefixView.appendChild(tr); 
			}
		}
	}
	
});

dojo.declare("profileEditor.Base", profileEditor.Profile, {
	
	templatePath: dojo.moduleUrl("profileTemplates","_Base.html"),

	startup: function(){
		this.connect(this.browseButton, "onClick", "_browseDir");
		
	},
	
	_browseDir: function(){
		var file = fileUtil.baseDir;
		var hand = this.connect(file, air.Event.SELECT, function(event) {
			this.disconnect(hand);
			fileUtil.baseDir = this._basePath = file;
			// problably need to sniff dojo.js, or check dojo/dojo.js 
			// and adjust accordingly based on user?
			var ds = file.resolvePath("dojo/dojo.js");
			var da = ds.exists ? ds : file.resolvePath("dojo.js");
			if(!da.exists){
				alert('unable to find a suitable dojo.js');
			}else{
				if(ds.nativePath != da.nativePath){
					// we actually need to adjust. 
					this._basePath = file = file.resolvePath("../");
				}
				this.baseDojoPath.value = file.nativePath;
				profileEditor._basePath = this._basePath; // cache it sorta
				this._scanProfiles();
			}
		});
		file.browseForDirectory("Where is the root of your Dojo directory?");
	},

	_loadProfile: function(){
		// var profile = this.profileSelect.options[this.profileSelect.selectedIndex].value;
		var profile = this.profileSelect.value; // gotta love safari
		air.trace('loading: '+profile);
	},
	
	_newProfile: function(){
		// the user explicityly clicked the "create / edit button
		var profile = new profileEditor.Profile({
			title: this.profileName.value,
			basePath: this._basePath
		});
		var pe = dijit.byId("profileEdit");
		pe.addChild(profile);
		pe.selectChild(profile);
		
		pe.removeChild(this);
		
		var tabs = pe.tablist.containerNode; 
		tabs.appendChild(dojo.byId("layerAddWrapper"));
		
		this.destroy();
	},
	
	_scanProfiles: function(){
		var pd = this._basePath.resolvePath("util/buildscripts/profiles");
		if(pd.exists){
			var profiles = dojo.map(pd.getDirectoryListing(), function(afile){
				return afile.name.match(/^\./) ? undefined : afile.name.replace(/\.profile\.js/,"");
			});
			
			if(profiles.length > 0){
				dojo.query("option", this.profileSelect).forEach(dojo._destroyElement);
				dojo.forEach(profiles, function(profile){
					if(profile){
						var opt = dojo.doc.createElement('option');
						opt.value = opt.innerHTML = profile;
						this.profileSelect.appendChild(opt);
					}
				}, this);
				
				// enable the select boxes, we know a path to work from
				this.profileName.disabled = false;
				this.profileUseNewRadio.disabled = false;
				this.profileSelect.disabled = false;
				this.profileSelectRadio.disabled = false;
				this.loadButton.setAttribute("disabled", false);
				this.createButton.setAttribute("disabled", false);

			}
			
			this._resourceTreeData = profileEditor.getModuleData();
		}
	},
	
	_setTitle: function(args){
		if(this["setTitle"]){
			this.setTitle.call(this, this.title);
		}
	}
});

dojo.declare("profileEditor.Layer", profileEditor.Profile, {
	// a layer for this profile instance, but it's just a list of tabs really
	templatePath: dojo.moduleUrl("profileTemplates","_Layer.html"),
	
	postCreate: function(){
		this.inherited(arguments);
		if(!this.resouceView){
			this.resouceView = new profileEditor.ResourceView({
				basePath: profileEditor._basePath 
			});
		}
		this.resourceHolder.appendChild(this.resouceView.domNode);
		
	},
	
	_handleChange: function(e){
		if(e && e.target){
			var t = e.target;
			if(t == this.profileName){
				this._setTitle();
			}
		}
	},
	
	_setTitle: function(args){
		if(this.profileName && this.profileName.value){
			this.title = this.profileName.value.replace(/[\W]*/g,"");
			if(args && args.append){
				this.title += args.append;
			}
		}
		this.inherited(arguments);
	}
	
});

dojo.declare("profileEditor.ResourceView", [dijit._Widget, dijit._Templated], {
	// a Tree view of the resources in this.basePath + prefixes.
	templatePath: dojo.moduleUrl("profileTemplates", "_ResourceView.html"),
	postCreate: function(){

		// fixme: doesn't f'ing work! not shown until after getModuleData() completes. 
		//activityBox.setMessage("Scanning Resources");
		//activityBox.show();
	
		this._treeData = profileEditor.getModuleData();
		
		// seems ok:
//		air.trace(this._treeData + " " + this._treeData.declaredClass);
		// FIXME: at this point, this data should be cached on disk 
/// 	air.trace(this._treeData.dump({
/// 		suppressExportMeta: true,
/// 		clone:true,
/// 		cleanMeta:true,
/// 		pretty:true // hacked in by me for sanity reasons
/// 	}));
///
 	
 	var model = new dijit.tree.ForestStoreModel({
			childrenAttrs: ["children"],
			query: "$[*]",
			store: this._treeData,
//			rootId:"ugh",
			rootLabel:"Resource Tree",
			labelAttr: "name"
		});
		
		var tree = new dijit.Tree({ model: model });

		this.containerNode.appendChild(tree.domNode);
		
//		tree.startup();

	}
});

profileEditor.scanTree = function(){
	
	var store = profileEditor._jsonStore = new dojox.data.jsonPathStore({
		data:{}, idAttribute:"id"
	 });
	
	var re = /\.js$/;
	var files = fileUtil.getFilteredFileList(profileEditor._basePath.nativePath, re, true);
	var resources = [];
	
	var reg = new RegExp("^" + profileEditor._basePath.nativePath +"/");
	dojo.forEach(files, function(file){
	
//		var data = fileUtil.readFile(file);
//		var provide = data.match(/dojo\.provide\s*\([\w\W]*?\)/g);
//		if(provide && provide[0]){
//			var foo = provide[0].match(/\"[d](.*)\"/);
//			if(foo && foo[1]){
//				resources.push(foo[1]);
//			}
//		}
//		
//	});
		var r = file.replace(reg,"");
//		air.trace(r);
//
//	dojo.forEach(resources,function(r){
		var curpath = [];	
		var parentItem = null;
		parts = r.split("/");
		for (var i = 0; i < parts.length; i++){
			curpath.push(parts[i]);
			var id = curpath.join("/");
			var item = store.byId(id);
			console.log("Item: ", item);
			if (!item){
				var newObj = {
					id: id, 
					name: parts[i], //,
					children: []
				}

				if (parentItem){
					item = store.newItem(newObj,{ parent: parentItem, attribute: "children" });
				}else{
					item = store.newItem(newObj);
				}
			}

			parentItem = item;
		}
		store.save();
	});

	return store; 
}

profileEditor.getModuleData = function(){

	if(!profileEditor._cache["modules"]){
		profileEditor._cache["modules"] = profileEditor.scanTree();
	}

	return profileEditor._cache["modules"];
}

var logger = air;

dojo.addOnLoad(function(){
	// summary: this plugins bootstrap now for:
	
	var theEditor = new toolbox.Module("profileEditor", {
		init: function(){

			dojo.parser.parse();

			var main = dijit.byId("profileEdit");
			var n = 1;
			var addb = dijit.byId("profileAdd");
			var tabs = main.tablist.containerNode;
			var rootPane = null;

			// janky hack, whenever we have a global path for a "working" dojo, use it here
			if(profileEditor._weKnowBase){
				rootPane = new profileEditor.Profile({ title:"Sample" });
			}else{
				rootPane = new profileEditor.Base({ title:"Profile Setup" });
			}
			main.addChild(rootPane);
			main.selectChild(rootPane);

			dojo.connect(addb, "onClick", function(){

				var child = new profileEditor.Layer({ 
					title: "Layer " + (n++),
					closable:true
				});

				main.addChild(child);
				main.selectChild(child);

				tabs.appendChild(dojo.byId("layerAddWrapper"));

			});

			main.layout();

		}

	});
	theEditor.init();
	
	
});