dojo.provide("toolbox.ApplicationContainer");
dojo.require("dijit.layout.StackContainer");
dojo.requireIf(!!window.air, "toolbox.utils");
dojo.require("dijit.form.Button");
dojo.require("dojo.dnd.move");
dojo.require("dojox.storage.AirFileStorageProvider");

dojo.declare("toolbox.ApplicationContainer", [dijit.layout.StackContainer, dijit._Templated], {

	topicPrefix: "/toolbox/",
	moduleDirectoryUrl: "app:/modules/",
	updateUrl: "http://download.dojotoolkit.org/toolbox/update.xml",
	isVertical: true,
 	moduleQuery: null,
	listWidget: null,
	moduleCache: null,
	isOpen: false,
	isOpening: false,
	commandLine: null,

	templatePath: dojo.moduleUrl("toolbox", "resources/ApplicationContainer.html?" + new Date().getTime()),
	widgetsInTemplate: true,

	/* widget lifecycle overrides */
	constructor: function() {

		this.moduleCache = {
			items: [],
			isDirty: true // you dirty dog
		};
		this.moduleQuery = { name: "*" };
		this.hoverTimer = null;	
		this.appStorage = new dojox.storage.AirFileStorageProvider();
		this.appStorage.DEFAULT_NAMESPACE = 'toolbox_appstorage';
		
	},
	
	publish: function(name, data){
		// summary: Localized publish mechanism
		dojo.publish(this.topicPrefix + name, [data || {}]);
	},

	initializeUpdater: function(){
	 	if(development){
			return;
		}
		
		var appUpdater = this.appUpdater = new runtime.air.update.ApplicationUpdaterUI();
		// start updater config  
		//change this to point to the online update.xml.
		appUpdater.updateURL = this.updateUrl;
		appUpdater.delay = 1;

		//by default all the dialogs are hidden
		appUpdater.isCheckForUpdateVisible = true;
		appUpdater.isDownloadUpdateVisible = true;
		appUpdater.isDownloadProgressVisible = true;
		appUpdater.isInstallUpdateVisible = true;

		// end updater config

		//it is necessary to add an ErrorEvent listener because this type of events are catched by the debugger
		appUpdater.addEventListener( runtime.flash.events.ErrorEvent.ERROR, function(ev){ air.trace(ev); } );

		appUpdater.initialize();
	},

	checkForUpdates: function(){
		console.log("Update check");
		if(development){
			return;
		}
		this.appUpdater.checkNow();
	},
	
	postCreate: function(){	
		this.inherited(arguments);
		
		this.btnlist = new toolbox.ToollistController({
			id: this.id + "_btnlist",
			containerId: this.id,
			_selfRef: this
		}, this.controllerNode);
		this.controllerNode = this.btnlist.domNode;
		dojo.byId("version").innerHTML = this._getVersion();
		
		dojo.connect(air.NativeApplication.nativeApplication, air.InvokeEvent.INVOKE, this, "onInvoke");
//		air.NativeApplication.nativeApplication.addEventListener(air.InvokeEvent.INVOKE, 
//				dojo.hitch(this, this.onInvoke));
				
		// set the initial Toolbox wherever is was last positioned
		var foo = this.appStorage.get("prefs_persists");
		if(foo){ 
			dojo.marginBox(this.domNode, foo.location); 
			this.setOrientation(foo.isVertical);
		}
	},
	
	closeWindow: function(){
		
		var openedWindows = air.NativeApplication.nativeApplication.openedWindows;
		// skip the first window, because that's the main toolbox window.
		// we kill the others first, then this one.
		for(var i=1; i < openedWindows.length; i++){
			openedWindows[i].close();
		}
		window.nativeWindow.close();
	},
	
	minimizeWindow: function(){
		window.nativeWindow.minimize();
	},
	
	_getVersion: function(){
		
		var xmlString = air.NativeApplication.nativeApplication.applicationDescriptor,
			appXml = new DOMParser(),
			xmlObject = appXml.parseFromString(xmlString, "text/xml"),
			root = xmlObject.getElementsByTagName('application')[0],
			appVersion = root.getElementsByTagName("version")[0];
			
		return appVersion.firstChild.data;
	},
	
	startup: function() {
		this.inherited(arguments);
		
		this.setOrientation(this.isVertical);
		this.btnlist.startup();
		
		// breathe before continuing
		setTimeout(
			dojo.hitch(this, function() {
//				console.log("init happens");
				this.publish("init");
			}),
			10
		);
	},
	
	initDataSources: function() {
//		console.log(this.id + ".initDataSources");

		// do we need to add load call back to the store to continue when its ready, 
		// or is it sync? 
	
		this.moduleMgr = toolbox.moduleManager.getInstance();
		this.moduleMgr.configure({
			pluginDirs: air.File.applicationDirectory.resolvePath("modules")
		});
		this.moduleMgr.update();
		this.moduleStore = this.moduleMgr.getModuleStore(); 

		// pre-fetch, cache the result so we can sync operations in some cases
		this.moduleStore.fetch({
			query: this.moduleQuery,
			start: 0,
			count: 25,
			onComplete: dojo.hitch(this, function(data) {
				this.moduleCache.items = data; 
				this.moduleCache.isDirty = false;
				// count isnt strictly accurate - in the (unlikely) case that we have >25 modules/plugsin installed
				this.moduleCount = data.length; 
				this.publish("dataready");
			})
		});
			
	},
	
	initUI: function() {
//		console.log(this.id + ".initUI");
		var panel = this.containerNode;
		
		// first use, the module cache should be safe to use
		var moduleItems = this.moduleCache.items;
		var store = this.moduleStore;
		dojo.forEach(moduleItems, function(item) {
			var props = {};
			if(this.moduleStore.isItem(item)) {
				props.title = store.getLabel(item);
				props.id = store.getIdentity(item);
				var iconSrc = store.getValue(item, "iconSrc");
				var width = store.getValue(item, "initialWidth");
				var height = store.getValue(item, "initialHeight");
				if(width) {
					props.initialWidth = width;
				}
				if(height) {
					props.initialHeight = height;
				}
				if(iconSrc) {
					props.iconSrc = dojo.moduleUrl("modules", item.id + "/" + iconSrc).toString();
				}
			} else {
				new Error("cached item isnt an item");
				return;
			}
			props.startUrl = dojo.moduleUrl("modules", props.id + "/" + props.id + ".html").toString();
			this.addChild( new toolbox.ModulePane(props) );
		}, this);
		
		// set up the hover event handling to open the toolbox tray
		this._timer = null;

		this.connect(this.wrapper, "onclick", "_toggle");
//		this.connect(this.wrapper, "onmouseover", "_enter");
//		this.connect(this.wrapper, "onmouseleave", "_leave");

		this.publish("uiready");

	}, 
	
	_toggle: function(e){
		if(this.isOpen && !this._appMoving){
			this.close();
		}else{
			if(!this.isOpening && !dojo.hasClass(dojo.body(),"dndMove")){
				this.open();
			}
		}
	},
	
	_enter: function(){
		// summary: handler function for mouse movement in the Toolbox
		if(this._timer){ clearTimeout(this._timer); }
		if(! (this.isOpen || this.isOpening )) {
			this.open();
		}
	},
	
	_leave: function(){
		// summary: handler function for mouse exiting the Toolbox
		if(this._timer){ clearTimeout(this._timer); }
		this._timer = setTimeout(dojo.hitch(this, "close", 475));
	},
	
	curtainUp: function() {
		// summary: post bootstrap, position ourselfs, and size out window.
		var rect;
		var windowBounds = new air.Rectangle(
			0, 0,
			// FIXME: we'll need to think of multi-head :) we all have more than one right? 
			air.Capabilities.screenResolutionX, air.Capabilities.screenResolutionY
		);
		this._screenRes = rect = window.nativeWindow.bounds = air.Screen.mainScreen.visibleBounds;

		// constrain the box to the screen/window
		var off = dojo.marginBox(this.wrapper);
		var boundaryBox = {
			l: rect.left + 2, 
			t: rect.top + 2, 
			w: rect.width - off.w - 2, 
			h: rect.height - off.h - 2
		};
		this._movable = new dojo.dnd.move.boxConstrainedMoveable(this.domNode, { 
//			handle: "controlSpace",
			box: boundaryBox
		});

		dojo.subscribe("/dnd/move/start", dojo.hitch(this, function(n){
			this._wasOpen = this.isOpen;
			this._appMoving = true;
			if(this._wasOpen){
				setTimeout(dojo.hitch(this,"close"), 100);
//				this.close();
			}
		}));

		dojo.subscribe("/dnd/move/stop", dojo.hitch(this, function(){
			// summary: Called after drop to determine if layout issues exist, and reorient as needed
			var pos = this._currentLocation = dojo.coords(this.domNode);
			var box = window.nativeWindow.bounds,
				 w = box.width,
				 h = box.height,		
				 rt = pos.l + off.w + this._openedOffset,
				 bt = pos.t + off.h + this._openedOffset
			;
			console.dir(pos);

			if(bt >= h){
				// FIXME: this is a quick hack, need to support All directions, etc,
				// and move this animation to show()
				dojo.anim(this.domNode,{
					top: pos.t - (bt - h)
				});
			}
//				if(this.isVertical){
//					this.setOrientation(false);
//				}
//			}else{
//				if(!this.isVertical){
//					this.setOrientation(true);
//				}
//			}
			
			this._expandDirection = "bottom";
//			if(!this.isVertical){
//				this._expandDirection = (rt >= w ? "left" : "right");
//				console.log('expand to: ' + this._expandDirection);
//			}else{
//				this._expandDirection = "bottom";
//			}

			this.appStorage.put("prefs_persists", {
				location: this._currentLocation,
				dir: this._expandDirection,
				isVertical: this.isVertical
			});
			
			this._appMoving = false;
			if(this._wasOpen){
				this.open();
				delete this._wasOpen;
			}

		}
		));
	
	},
	
	setOrientation: function(isVertical) {
		// summary: Toggle the layout to become vertical or horizontal
		isVertical = this.isVertical = !!isVertical;
		if(isVertical) {
			dojo.removeClass(this.domNode, "layoutH");
			dojo.addClass(this.domNode, "layoutV");
		}else{
			console.log(this._expandDirection);
			dojo.removeClass(this.domNode, "layoutV");
			dojo.addClass(this.domNode, "layoutH");
		}
		console.log("setOrientation: isVertical: %s, className: %s", isVertical, this.domNode.className);
	},

	open: function() {
		// summary: grow the container to/and show list contents
		var showMax = 10;

		if(this._openAnim || this._appMoving) { return; }
		if(this._closeAnim) { this._closeAnim.stop(); }
		
		var node = this.controllerNode, 
			numPlugins = this.moduleCount;
		
		var dim = this.isVertical ? "height" : "width";
		
		var endProps = {}, 
			otherEnd = {},
			offset = 0;

		// grow the container to show max 10 icons
		// FIXME: adjust for width of horizontal items. 80 is the height for vertical
		this._openedOffset = endProps[dim] = (numPlugins * 70) + offset;
		otherEnd[dim] = endProps[dim] + (this.isVertical ? 121 : 275);

		dojo.style(node, "overflow", "hidden");

		var wrapAnim = dojo.animateProperty({
			node: this.wrapper, 
			properties: otherEnd,
			duration:150
		});
		
		var anAnim = dojo.animateProperty({
			node: node,
			properties:endProps,
			duration:150
		});
		
		this._openAnim = dojo.fx.combine([wrapAnim, anAnim]);
		this.connect(this._openAnim,"onEnd", function(){
				this.isOpening = false; 
				this.isOpen = true; 
				delete this._openAnim;
		});
		
		this.isOpening = true;
		this._openAnim.play();
	},

//	_closedSize: function(){
//		var v = this.isVertical;
//		return {
//			"height" : v ? 145 : 145,
//			"width" : v ? 145 : 145
//		}
//	},
//	
	close: function() {

		this.isClosing = true;
//		dojo.style(this.wrapper, dim, "145px");
		// TODO: handle orientation changes
	//	this._openAnim && this._openAnim.stop();
		
		var dim = this.isVertical ? "height" : "width";
		var endProps = {};
		endProps[dim] = 0;
		var otherEnd = {};
		otherEnd[dim] = this.isVertical ? 145 : 175; //this._closedSize()[dim];
//		if(this._openDirection == "left"){
//			
//		}
		var node = this.controllerNode;
		
		var anAnim = dojo.animateProperty({
			node: node,
			properties: endProps,
			duration:150
		});
		
		var otherAnim = dojo.animateProperty({
			node: this.wrapper,
			properties: otherEnd,
			duration:150
		});
		
		this._closeAnim = dojo.fx.combine([anAnim, otherAnim]);
		dojo.connect(this._closeAnim, "onEnd", this, function(e){
				this.isClosing = false; 
				this.isOpen = false; 
				delete this._closeAnim;
		});
	
		this._closeAnim.play();
	},
	
	addChild: function(/*Widget*/ child, /*Integer?*/ insertIndex){
		// console.log(this.declaredClass + " addChild, got child of type: " + child.declaredClass);
		// summary: Adds a widget to the stack
		 
		dijit._Container.prototype.addChild.apply(this, arguments);
		child = this._setupChild(child);

		if(this._started){
			// console.log(this.declaredClass + " addChild, publishing -addChild");
			dojo.publish(this.id+"-addChild", [child, insertIndex]);
		}
	},

	removeChild: function(/*Widget*/ page){
		// summary: Removes the pane from the stack

		dijit._Container.prototype.removeChild.apply(this, arguments);

		// If we are being destroyed than don't run the code below (to select another page), because we are deleting
		// every page one by one
		if(this._beingDestroyed){ return; }

		if(this._started){
			// this will notify any tablists to remove a button; do this first because it may affect sizing
			dojo.publish(this.id+"-removeChild", [page]);

			// in case the tab titles now take up one line instead of two lines
			this.layout();
		}

		if(this.selectedChildWidget === page){
			this.selectedChildWidget = undefined;
		}
	},
	
	_setupChild: function(/* Widget */pane){
		dojo.addClass(pane.domNode, "modulePaneWrapper");
		this.inherited(arguments);
		return pane; // Widget
	},
	
	_showChild: function(/*Widget*/ page){
		var children = this.getChildren();
		page.isFirstChild = (page == children[0]);
		page.isLastChild = (page == children[children.length-1]);
		page.selected = true;
		page.show();
		if(page._loadCheck){
			page._loadCheck(); // trigger load in ContentPane
		}
		if(page.onShow){
			page.onShow();
		}
	},
	
	onInvoke: function(/*InvokeEvent*/ event){
		if(this.getChildren().length > 1){
			this.processCommandLine(event.arguments);
		}else{
			setTimeout(dojo.hitch(this, this.onInvoke, event), 150);
		}
	},
	
	processCommandLine: function(/*Array*/ args){
		/* Process command line arguments. You can pass a 
		<command> <tool id> [options]
		command line. <command> can be "open" currently. */
		
		if(args.length < 2){
			return;
		}
		if(args[0] != "open"){
			air.trace("Command option can currently only be 'open'");
			return;
		}
		var toolId = args[1];
		var children = this.getChildren();
		var tool = null;
		for(var i = 0; i < children.length; i++){
			if(children[i].id == toolId){
				tool = children[i];
			}
		}
		if(!tool){
			air.trace("Could not find tool with ID: " + toolId);
			return;
		}
		var remainder = args.slice(2);
		
		tool.show(remainder);
	}

});

dojo.declare("toolbox.ToollistController", dijit.layout.StackController,
	{
	// summary:
	// 	Set of tabs (the things with titles and a close button, that you click to show a tab panel).
	// description:
	//	Lets the user select the currently shown pane in a TabContainer or StackContainer.
	//	TabController also monitors the TabContainer, and whenever a pane is
	//	added or deleted updates itself accordingly.
	containerId: "",
	listClass: "toolboxList",
	templateString: '<div wairole="tablist" dojoAttachPoint="domNode,containerNode" dojoAttachEvent="onkeypress:onkeypress"  class="toolboxList ${listClass}"></div>',

	// buttonWidget: String
	//	The name of the tab widget to create to correspond to each page
	buttonWidget: "toolbox.ToollistButton",

	onAddChild: function(/*Widget*/ page, /*Integer?*/ insertIndex) {
		// console.log(this.declaredClass + ".onAddChild got page of class: " + page.declaredClass);

		// summary:
		//   Called whenever a page is added to the container.
		//   Create button corresponding to the page.

		// create an instance of the button widget
		var button = new toolbox.ToollistButton({
				iconSrc: page.iconSrc,
				title: page.title, 
				closeButton: page.closable, 
				itemId: page.itemId,
				id: page.id + "_btn"
		});
		this.domNode.appendChild(button.domNode);

		this.addChild(button, insertIndex);

		this.pane2button[page] = button;
		page.controlButton = button;	// this value might be overwritten if two tabs point to same container
		
		dojo.connect(button, "onClick", dojo.hitch(this, "onButtonClick", page));
		
		if(page.closable){
			dojo.connect(button, "onClickCloseButton", dojo.hitch(this, "onCloseButtonClick", page));
		}
		if(!this._currentChild){ // put the first child into the tab order
			button.focusNode.setAttribute("tabIndex", "0");
			this._currentChild = page;
		}
		//make sure all tabs have the same length
		if(!this.isLeftToRight() && dojo.isIE && this._rectifyRtlTabList){
			this._rectifyRtlTabList();
		}

		// console.log(this.declaredClass + ".onAddChild worked out fine");
	},

	onRemoveChild: function(/*Widget*/ page){
		// summary:
		//   Called whenever a page is removed from the container.
		//   Remove the button corresponding to the page.
		if(this._currentChild === page){ this._currentChild = null; }
		var button = this.pane2button[page];

		if(button){
			// TODO? if current child { reassign }
			button.destroy();
		}
		this.pane2button[page] = null;
	},
	
	onButtonClick: function(/*Widget*/ page){
		// summary:
		//   Called whenever one of my child buttons is pressed in an attempt to select a page
		page && page.show && dojo.isFunction(page.show) && page.show();
	}
	
});

dojo.declare("toolbox.ToollistButton",
	//[dijit._Widget, dijit._Templated, dijit._Contained],
	dijit.form.ToggleButton,
	{
	// summary:
	//	A button (the thing you click to select a pane).
	// description:
	//	Contains the title of the pane, and optionally a close-button to destroy the pane.
	//	This is an internal widget and should not be instantiated directly.

	baseClass: "toolboxItem",
	itemClass: "",
	title: "Untitled",
	name: "",
	iconSrc: dojo.moduleUrl("toolbox", "resources/defaultToolIcon.png"),

	tabIndex: "-1", // StackContainer buttons are not in the tab order by default

	templatePath: dojo.moduleUrl("toolbox", "resources/ToollistButton.html"),
	
	postMixInProperties: function() {
		this.inherited(arguments);
		if(this.name && this.title === dojo.getObject(this.declaredClass).prototype.title) {
			this.name = this.title;
		}
	},
	postCreate: function(){
		this.inherited(arguments); 
		dojo.setSelectable(this.domNode, false);
	},
	
	onClick: function(/*Event*/ evt){
		// summary: This is for TabContainer where the tabs are <span> rather than button,
		// 	so need to set focus explicitly (on some browsers)
//		console.log("onClick");
		dijit.focus(this.focusNode);
		// ... now let StackController catch the event and tell me what to do
	},
	
	onClickCloseButton: function(/*Event*/ evt){
		// summary
		//	StackContainer connects to this function; if your widget contains a close button
		//	then clicking it should call this function.
		evt.stopPropagation();
	}
	
});
