dojo.provide("toolbox.FileInputComboBox");

dojo.require("dijit.form.ComboBox");
dojo.require("dijit.form.Button");

dojo.declare("toolbox.FileInputComboBox", [dijit._Widget, dijit._Templated], {
	widgetsInTemplate: true,
	label: "Browse...",
	prompt: "",
	templateString: null,
	width: "100%",

	id: "",
	name: "",
	
	templatePath: dojo.moduleUrl("toolbox", "resources/FileInputComboBox.html"),
	// extensions:
	// 	";" delimited list of file extensions to filter for in the open dialog
	// 		e.g. "*.as;*.css;*.html;*.txt;*.xml"
	extensions: "",
	
	browseMode: "file", // or "directory"
	// pageSize: Integer
	//		Argument to data provider.
	//		Specifies number of search results per page (before hitting "next" button)
	pageSize: Infinity,

	// store: Object
	//		Reference to data provider object used by this ComboBox
	store: null,

	// query: Object
	//		A query that can be passed to 'store' to initially filter the items,
	//		before doing further filtering based on `searchAttr` and the key.
	//		Any reference to the `searchAttr` is ignored.
	query: {},
	
	// identifier: String
	// the attribute name in that's used as the identifier in the store's items
	identifier: "",

	require: false, 
	isValid: function(arg) {
		return this.validator.call(this, arg || this.comboBox.getValue());
	},
	validator: function(arg, constraints) {
		var value = arg || this.comboBox.getValue();
		if(!dojo.isObject(constraints)) {
			constraints = { exists: true };
		}
		if(value) {
			var file = new air.File(value);
			for(var key in constraints) {
				if(file[key] != constraints[key]) {
					return false;
				}
			}
			return true;
		} else {
			return false;
		}
	},
	invalidMessage: "bad filename",

	// autoComplete: Boolean
	//		If you type in a partial string, and then tab out of the `<input>` box,
	//		automatically copy the first entry displayed in the drop down list to
	//		the `<input>` field
	autoComplete: true,

	// highlightMatch: String
	// 		One of: "first", "all" or "none".
	//		If the ComboBox opens with the serach results and the searched
	//		string can be found it will be highlighted.
	//		This value is not considered when labelType!="text" to not
	//		screw up any mark up the label might contain.
	highlightMatch: "first",

	// searchDelay: Integer
	//		Delay in milliseconds between when user types something and we start
	//		searching based on that value
	searchDelay: 100,

	// searchAttr: String
	//		Searches pattern match against this field
	searchAttr: "name",

	// labelAttr: String
	//		Optional.  The text that actually appears in the drop down.
	//		If not specified, the searchAttr text is used instead.
	labelAttr: "",

	// labelType: String
	//		"html" or "text"
	labelType: "text",

	// queryExpr: String
	//		dojo.data query expression pattern.
	//		`${0}` will be substituted for the user text.
	//		`*` is used for wildcards.
	//		`${0}*` means "starts with", `*${0}*` means "contains", `${0}` means "is"
	queryExpr: "${0}*",

	// ignoreCase: Boolean
	//		Set true if the ComboBox should ignore case when matching possible items
	ignoreCase: true,
	
	postMixInProperties: function() {
		if(!this.prompt) {
			this.prompt = (this.browseMode == "file") ? "Select File" : "Select Directory"; 
		}
	},

	postCreate: function() {
		this.inherited("postCreate", arguments); 
		this.focusNode = this.comboBox.focusNode; 

		// copy values over to the comboBox
		var combo = this.comboBox;
		var pnames = [
			"id", "name", 
			"pageSize", 
			"store", 
			"autoComplete", "highlightMatch", "searchDelay", "searchAttr", "ignoreCase",
			"labelAttr", "labelType", 
			"queryExpr", 
			"required", "promptMessage", "invalidMessage"
		];
		dojo.forEach(pnames, function(key) {  
			combo[key] = this[key];
		}, this); 
		combo.isValid = dojo.hitch(this, function() {
			return this.isValid();
		});
		combo.query = this.query;
		
		// TODO: handle enter key scenario, where the user is entering in a new value
		this.connect(this.browseButton, "onClick", "_onBrowseClick");
		
		this._showResultList
	},
	startup: function() {
		this.comboBox.onChange = dojo.hitch(this, function() {
			this.onChange.apply(this, arguments);
		});
		this.inherited("startup", arguments); 
	},
	_askForFilePath: function(deferred) {
		var params = {
			prompt: this.prompt
		};
		var file = new air.File();
		var filters = [];
		if(this.extensions) {
			filters.push( new air.FileFilter("Files", this.extensions) );
		}
		try {
				file.browseForOpen(params.prompt, filters);
		    file.addEventListener(air.Event.SELECT, function(event) {
			    deferred.callback(event.target);
				});
		} catch (error) {
		    console.log("select File failed:", error.message);
				deferred.errback(error);
		}
	},
	_askForDirectoryPath: function(deferred) {
		params = {
			prompt: this.prompt
		};
		var dir = new air.File();
		try {
				dir.browseForDirectory(params.prompt);
		    dir.addEventListener(air.Event.SELECT, function(event) {
			    deferred.callback(event.target);
				});
		} catch (error) {
		    console.log("selectDir failed:", error.message);
				deferred.errback(error);
		}
	},
	addItem: function(path, parentInfo) {
		var store = this.store, 
			comboBox = this.comboBox,
			identifier = this.identifier; 
		var itemProps = {};

		console.log("making new item with identifier: " + path);
		itemProps[identifier] = path;

		if(this.labelAttr){
			// use the file/directory name as the label
			// itemProps[this.comboBox.labelAttr] = path.split(/\/|\\/).pop();
			itemProps[this.comboBox.labelAttr] = path;
		}
		if(this.searchAttr){
			// the search attribute should be the full path for the right auto-completion behavior
			itemProps[this.comboBox.searchAttr] = path;
		}

		// create the new item if the store supports it
		if(store.getFeatures()["dojo.data.api.Write"]) {
			// TODO: any parentInfo we can supply??
			var item = this.item = store.newItem(itemProps, parentInfo);
			store.save();
			// this.setValue(store.getValue(item, comboBox.searchAttr), true);
		}else {
			console.warn(store.declaredClass + " doesnt support newItem, cant create new item for " + path);
		}
	},
	_onBrowseClick: function() {
		var store = this.store, 
			comboBox = this.comboBox;
			
		// getting the path will be an asynchronous operation
		var deferred = new dojo.Deferred();
		var identifier = this.identifier; 
		if(!identifier) {
			identifier = this.identifier = (store.getFeatures()["dojo.data.api.Identity"]) ? store.getIdentityAttributes()[0] : "";	
		}

		deferred.addCallback(
			dojo.hitch(this, function(fileObj) {
				
				var path = fileObj.nativePath;

				var onError = dojo.hitch(this, function(err) {
					console.warn("error looking up: ", path, err);
				});
				var self = this;
				var onItem = dojo.hitch(self, function(item) {
					var identifier = this.identifier;
					if(item && !(item instanceof Error)) {
						// some stores give an Error when lookup fails, some just return null
						this.comboBox.item = item;
						this.setValue(store.getValue(item, comboBox.searchAttr), true);
					} else {
						/// validate it, and either add the new item, or just show the invalid value
						if(this.isValid(path)) {
							this.addItem(path);
						}
						this.setValue(path, true);
					}
				});

				store.fetchItemByIdentity({
					identity: path, 
					onItem: onItem,
					onError: onError
				});
			})
		);
		deferred.addErrback(
			function(err) {
				console.log("bad result:" + err.message);
			}
		);

		if(identifier && typeof identifier == "string") {
				// doit
				if(this.browseMode == "directory") {
					this._askForDirectoryPath(deferred);
				} else {
					this._askForFilePath(deferred);
				}
		} else {
				console.warn("no identifier: " + identifier + ", bailing");
		} 
	},
	getValue: function(){
		return this.comboBox.getValue();
	},
	setValue: function(/*Object*/ value){
		// if the store allows it, persist the value
		this.comboBox.setValue(value);
	},
	onChange: function() {
		console.log("current value: " + this.getValue());
	}
});

