dojo.provide("toolbox.ModuleList");
dojo.require("dijit.dijit");

console.log("started loading ModuleList");

dojo.declare("toolbox.ModuleList", [dijit._Widget, dijit._Templated], {
	start: 0, 
	fetchMax: 25,
	defaultValue: "novalue",
	
	// autoload: Boolean
	// 		should load and display data at creation time? 
	autoload: false,
	
	moduleBasePath: "app:/modules",
	
	/* css classes */
	listClass: "list", 
	itemClass: "item", 
	
	/* Store: dojo.data-based store object */
	store: null, 
 	query: { name: "*" },

	templateString: '<div class="toolboxList ${listClass}"><ul dojoAttachPoint="containerNode"></ul></div>', 
	itemTemplateString: '<li class="toolboxItem ${itemClass}" itemId="${id}">'
	+	'<img class="icon" width="32" height="32" src="${iconSrc}"/>'
	+	'<div class="label">${name}</div>'
	+'</li>', 

	constructor: function() {
		var wc = dojo.getObject(this.declaredClass);

		// build the list of attributes we'll want from the store
		// by scanning the item template
		this.attrs = dojo.map(
			wc.prototype.itemTemplateString.match(/\$\{([^\s\:\}]+)(?:\:([^\s\:\}]+))?\}/g),
			function(s){ return s.slice(2, -1); }
		);
	},

	postCreate: function() {
		if(this.autoload) {
			this.update();
		}
		this.connect(this.containerNode, "onclick", "_onContainerClick");
	}, 
	_onContainerClick: function(evt) {
		var node = evt.target; 
		if(node != this.containerNode) {
			
		}
		while(node && node.tagName !=="LI") {
			node = node.parentNode;
		}
		this.onItemClick({
			id: dojo.attr(node, "itemId")
		});
	},
	onItemClick: function(evtData) {
	 	// stub
	},

	clear: function(){
		var ctr = this.containerNode; 
		while(ctr.firstChild) {
			dojo._destroyElement(ctr.firstChild);
		}
		// and remove any state/local data
	},

	update: function(){
		this.fetch();
	},
	
	destroy: function() {
		if(this._tmpContainer) {
			this._tmpContainer = null; 
			delete this._tmpContainer;
		}
		this.inherited(arguments);
	},

	_addItem: function(/*Object*/data, idx){
		if(dojo.isString(data)){
			data = { value: data };
		}
		data.iconSrc = [this.moduleBasePath, data.id, data.iconSrc].join("/");
		if(!this._tmpContainer) {
			this._tmpContainer = dojo.doc.createElement("ul"); 
		}
		this._tmpContainer.innerHTML= dojo.string.substitute(this.itemTemplateString, data);
		var itemNode = this._tmpContainer.firstChild; 
		dojo.place(itemNode, this.containerNode, idx);
	},

	getAttrValuesObj: function(item){
		var obj = {};
		if(dojo.isString(item)){
			dojo.forEach(this.attrs, function(attr){
				obj[attr] = (attr == "value") ? item : this.defaultValue;
			}, this);
		}else{
			dojo.forEach(this.attrs, function(attr){
				if(this.store){
					obj[attr] = this.store.getValue(item, attr)||this.defaultValue;
				}else{
					obj[attr] = item[attr]||this.defaultValue;
				}
			}, this);
		}
		return obj;
	},

	onDataAvailable: function(data){
		this.clear();
		// console.log(data);
		dojo.forEach(data, function(item, idx){
			this._addItem(this.getAttrValuesObj(item), idx);
		}, this);
	},

	fetch: function(query, start, end){
		this.store.fetch({
			query: query||this.query,
			start: start||this.start,
			count: end||this.fetchMax,
			onComplete: dojo.hitch(this,"onDataAvailable")
		});
	}
});
console.log("end loading ModuleList");
		
