dojo.provide("toolbox.ShrinkSafeRequest"); 

// dojo.require("dojox.io.xhrMultiPart");
dojo.require("dojox.uuid.generateRandomUuid");

/* 
	gather an array of files
	create a hidden form with inputs for each
*/

dojo.declare("toolbox.ShrinkSafeRequest", null, {
	/*
		transpart: String
			http method to use
	*/
	transport: "POST",
	/* 
		target: String
			url the service can be reached at
	*/
	target: "",
	dataFormat: "text", // one of "text", "binary", "variables" as defined by air.URLLoaderDataFormat
	baseDirectory: null,
	fileParamName: "shrinkfile[]",
	options: null,
	
	constructor: function(args) {

		// initialize the mutable properties
		
		// an array of all the air.File objects we'll be sending
		this._files = []; 
		
		// a map of other name:value options
		this.options = dojo.mixin(
			{stripnewlines: 0}, 
			dojo.mixin(this.options || {})
		);
		
		// mixin the args
		dojo.mixin(this, args);
		
		// allow a handleAs alias for dataFormat
		if(this.handlAs && (this.dataFormat == toolbox.ShrinkSafeRequest.prototype.dataFormat)) {
			this.dataFormat = this.handlAs;
		}
		// re-cast any string values to the AIR objects we'll be using
		if(typeof this.baseDirectory === "string") {
			this.baseDirectory = new air.File(this.baseDirectory);
		}
		if(!this.baseDirectory) {
			this.baseDirectory = air.File.applicationDirectory; 
		}
	},
	setOption: function(name, value) {
		if(typeof value !== "undefined" && value !== null) {
			this.options[name] = value;
		} else {
			delete this.options[name];
		}
	},
	
	addFile: function(filename) {
		var file = (filename.nativePath) ? filename : this.baseDirectory.resolvePath(filename);
		if(file.exists) {
			return this._files.push(file);
		} else {
			return false;
		}
	},
	/*=====
	dojox.io.__xhrContentArgs = function(){
		//	name: String
		//		Name of the form value.
		//	content: String
		//		The contents of the value.
		//	filename: String?
		//		An optional filename to pass to the server, as defined by the boundary.
		//	contentType: String?
		//		An optional content-type (MIME) to pass to the server, if value is being
		//		treated as a file.
		//	charset: String?
		//		Optional charset to pass, for the server to interpret the file correctly.
		//	contentTransferEncoding: String?
		//		Optional transfer encoding header value.
		this.name = name;
		this.content = content;
		this.filename = filename;
		this.contentType = contentType;
		this.charset = charset;
		this.contentTransferEncoding = contentTransferEncoding;
	}
	=====*/
	_createPart: function(/* dojox.io.__xhrContentArgs */args, /* String */boundary){
		//	summary
		//		Assemble an array of boundary parts based on the passed values in args.
		if(!args["name"] && !args["content"]){
			throw new Error("Each part of a multi-part request requires 'name' and 'content'.");
		}

		var tmp = [];
		tmp.push(
			"--" + boundary,
			 "Content-Disposition: form-data; name=\"" + args.name + "\"" + (args["filename"] ? "; filename=\"" + args.filename + "\"" : "")
		);

		if(args["contentType"]){
			var ct = "Content-Type: " + args.contentType;
			if(args["charset"]){
				ct += "; Charset=" + args.charset;
			}
			tmp.push(ct);
		}

		if(args["contentTransferEncoding"]){
			tmp.push("Content-Transfer-Encoding: " + args.contentTransferEncoding);
		}
		tmp.push("", args.content);
		return tmp;		//	Array
	},

	_prepareRequest: function() {
		var parts = [], 
			fieldname = this.fileParamName, 
			out="";
			
		// unique guid as a boundary value for multipart posts
		var boundary = this._boundary =dojox.uuid.generateRandomUuid();

		// make a args object to represent each part in the request
		dojo.forEach(this._files, function(file){
			var contentArg = {
				name: fieldname,
				content: toolbox.utils.readFile(file),
				filename: file.name // TODO: use relative filename? 
				// contentType = "contentType";
				//charset = charset;
				//contentTransferEncoding = contentTransferEncoding;
			};
			parts = parts.concat( this._createPart(contentArg, boundary) );
		}, this);
		
		// tack on other parameters
		var empty = {};
		for(var key in this.options) {
			if(! (key in empty) ) {
				parts = parts.concat(this._createPart({
					name: key,
					content: this.options[key]
				}, boundary));
			}
		}

		if(parts.length){
			parts.push("--"+boundary+"--", "");
			out = parts.join("\r\n");
		}
		
		var request = new air.URLRequest(this.target);
		request.contentType = 'multipart/form-data; boundary='+boundary;
		request.method = air.URLRequestMethod.POST;
		request.data = out; 
		
		return request;
	},
	
	send: function(req) {
		// some last-minute normalization
		var fileParamName = this.fileParamName.replace(/[\[\]]/g, ""); 
		// we send the fileParamName through to the server
		this.options.fileParamName = fileParamName; 

		var req = req || this._prepareRequest();
		var loader = this._loader = new air.URLLoader();

		var proto = dojo.getObject(this.declaredClass).prototype; 
		var isImplemented = dojo.hitch(this, function(method) {
			return proto[method] !== this[method];
		});
		// only register for some events if the stubs are implemented/overloaded
		if(isImplemented("onProgress")){
			loader.addEventListener("progress", dojo.hitch(this, this.onProgress));
		}

		loader.addEventListener("ioError", dojo.hitch(this, function(err) {
			this.onError(err, req);
		}));
		loader.addEventListener("complete", dojo.hitch(this, function() {
			this.onLoad(this._loader.data, req);
		}));

		// do it
		loader.load(req);
		return loader; // air.URLLoader
	}, 
	abort: function() {
		if(this._loader) {
				loader.close();
		}
	},
	onLoad: function(data, loader) {
		// stub
		console.log(this.declaredClass + ".onLoad"); 
	},
	onError: function(err, loader) {
		// stub
		console.log(this.declaredClass + ".onError"); 
		console.log(err.message); 
	},
	onProgress: function(evt, loader) {
		// stub
	}
});