dojo.provide("toolbox.core");

dojo.require("dojo.parser");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.layout.AccordionContainer");
dojo.require("dojox.fx.easing");
dojo.require("dijit.form.Button");

dojo.require("dojox.data.jsonPathStore");

dojo.require("toolbox.ApplicationContainer");
dojo.require("toolbox.ModulePane");

dojo.registerModulePath("modules", "../modules");

// dojo.require("toolbox.ModuleList");
if(typeof air === "undefined") {
	air = false;
} else {
	dojo.require("toolbox.moduleManager");
}

(function() {
	
	/* 
	init sequence:

	at pageload/DOMLoaded, the parser runs
	ApplicationContainer is created
	"init" topic fires
	    * plugins are discovered and loaded
	    * plugin data store is created and populated

	"dataready" topic fires - time to populate the UI
	    * ApplicationContainer's initUI runs
	    * which populates the list (tool tray) with icons/buttons
	    * and prepares containers for each tool

	"uiready" topic fires - the UI becomes interactive

	User interactions:

	mouseenter on the app opens the tool tray
	mouseout on the app closes the tool tray (if no tool panels are open)

	click on a tool button in the tray (open panel):
	    * any open panel is closed
	    * button is visibly active
	    * panel slides out with the plugin's content framed in a titlebar/footerbar container
	    * a window is created, bound on first use to house the plugin content
	    * plugin is sent a onOpen event

	panel is closed by user:
	    * button made inactive
	    * plugin is sent onClose event
	    * no explicit measures taken to unload or reset state on the plugin
	    * the window is not recovered / reused

	plugin discovery:
	    * We can programatically add a directory to be scanned for plugins
	    * By default, we'll check the appliation storage directory/plugins and the application (install) directory
	    * Any .zip files (or extension of our choice) found will be expanded
	    * All directories will be examined for a plugin.json file - metadata for the plugin
	    * Conforming directories will be registered as plugins, and added to the plugin datastore
 	
	*/
	
	toolbox.core.bindApp = function() {
		dojo.subscribe("/toolbox/init", app, function(evt) { 

			// check for updates
			app.initializeUpdater();

			// define dimensions for the application window
			// could do detection, sizing here
			console.log("setting moduleWindowProperties");
			app.windowBox = {
				w: 800,
				h: 600
			}; 

		});
		dojo.subscribe("/toolbox/init", app, function(evt) { app.initDataSources() } );

		dojo.subscribe("/toolbox/dataready", app, function(evt) { app.initUI() } );
		dojo.subscribe("/toolbox/uiready", app, function(evt) { 
			app.curtainUp();
		} );
		
		if (air.NativeApplication.supportsMenu) {
			var menuitem = new air.NativeMenuItem("Check for Updates...");
			menuitem.addEventListener(air.Event.SELECT, 
										dojo.hitch(app, app.checkForUpdates));
			var menu = air.NativeApplication.nativeApplication.menu.getItemAt(0);
			menu.submenu.addItemAt(menuitem, 1);
		}
		
		air.NativeApplication.nativeApplication.addEventListener(air.Event.EXITING,
			app.closeWindow);
	}

})();

