dojo.provide("toolbox.moduleManager"); 
dojo.require("toolbox.utils");
dojo.require("dojo.data.ItemFileWriteStore"); 

toolbox.moduleManager = {};

(function() {

	var moduleManager = new function() {
		/* 
			summary: 
				module housekeeping: loads, installs, and provides metadata on modules
		*/
		this._pluginDirs = []; 
		this.store = null;
		this._storePathMap = {};
		console.log("in moduleManager constructor");
	};

	toolbox.moduleManager.getInstance = function() {
		console.log("moduleManager singleton has configure method: " + typeof moduleManager.configure);
		return moduleManager;
	};
	console.log("mixing in properties to moduleManager singleton");

	dojo.mixin(moduleManager, {
	/*
		get module - we pass it the url (file:/ or http:// etc.) to a .json descriptor
		(kind of like ppm)
		which contains a resourceUrl

	*/	
		// applicationStorageDirectory	A place to store files needed by the application, such a settings or log files.
		// applicationResourceDirectory	The application's installation directory.
		// we assume the AIRAliases is loaded, to give us the "air" object
		moduleDirectory: air.applicationResourceDirectory,
		propertiesFilename: "properties.json",
		identifier: "id",

		pluginExtension: ".zip",
		
		init: function() {
			// register pluginDirs passed to the ctor in our must-be-unique map
			this._initializeStore();
			console.log("MM.init:  store item identifier: " + moduleManager.store._jsonData.identifier);
		},
		
		_initializeStore: function(params) {
			params = dojo.mixin(params || {}, {
				data: {
					identifier: "id", 
					labelAttr: "name",
					label: "name",
					// initialize as empty
					items: []
				}
			});
			this.store = new dojo.data.ItemFileWriteStore( params );
			return this.store;
		},
		
		configure: function(config) {
			// we need a module directory at minimum
			if(!config) { config = {}; }

			if(config.pluginDirs) {
				// ensure we've got an array to loop over
				if(!dojo.isArray(config.pluginDirs)) {
					config.pluginDirs  = [ config.pluginDirs ];
				}
				// wipe the existing array
				this._pluginDirs = [];
				
				console.log("configure: got a pluginDirs to add");
				console.log(config.pluginDirs[0]);
				dojo.forEach(config.pluginDirs, this.addPluginDir, this);
				delete config.pluginDirs; 
			}
			var filenames = dojo.map(this._pluginDirs, function(file) {
				return file.nativePath
			});
			dojo.mixin(this, config);
			console.log("MM-configure has pluginDirs: " + filenames.join("\n"));
			console.log("MM-configure has identifier: " + moduleManager.store._jsonData.identifier);
		},
		addPluginDir: function(dir) {
			if(typeof dir !== "object") {
				dir = new air.File(dir);
			}
			if(this._storePathMap[dir.nativePath]) {
				console.warn("directory: " + dir.nativePath + " already exists in _pluginDirs");
			} else {
				console.log("configure: adding dir: " + dir);
				this._pluginDirs.push( dir );
				console.log("configure: which has path: " + dir.nativePath);
			}
		},
		getPluginDirs: function(dir) {
			return this._pluginDirs;
		},

		update: function(/* Boolean */force) {
			console.log("MM-update");
			/*
				look in the plugin directorys
			 	plugins register themselves as they are loaded, 
				so any un-registered plugins are loaded

				2 passes: 
					first pass is to expand any .zip files (or whatever extension we use)
					2nd pass is to peek inside each directory for the descriptor/config .json file

			*/
			
			var pluginDirs = [].concat(this.getPluginDirs());
			// dojo.forEach(pluginDirs, this._expandZipsInDir, this);
			if(force) {
				// un-load plugins (all???) and re-load them
			} else {
				dojo.forEach(pluginDirs, function(dir) {
					// console.log("calling _loadPluginsInDir on path: " + dir.nativePath);
					this._loadPluginsInDir(dir);
				}, this);
			}
		},
		getModuleStore: function() {
			return this.store;
		},
		onNew: function(item) {
			// console.log("my onNew got item: " + item.id);
		},
		_expandZipsInDir: function(/*air.File*/dir) {
			// NOTE: not in use
			if(!dir.isDirectory) {
				return;
			}
			var reExt = new RegExp('.\\.zip' + "$"); 
			dojo.forEach(
				dojo.filter(
					dir.getDirectoryListing(), 
					function(file) {
						return reExt.test(file.nativePath);
					}
				), 
				function(file) {
					var zipHandler = new toolbox.utils.Zip(file); 
					zipHandler.expandTo(dir);
				}, 
				this
			);
			// TODO: remove the .zip file when done
		},
		_loadPlugin: function(/*air.File*/pluginDir, props) {
			// plugins are discovered and registered at startup - not persisted
			// and the store created anew each time the app runs
			// console.log("_loadPlugin: from: "  + pluginDir.url + " with props:" + props);
			
			var moduleManager = this;

			// create the data object to create an item from
			var itemData = dojo.mixin({
				directoryUrl: pluginDir.url
			}, props); 
			if(!itemData.id) {
				itemData.id = pluginDir.nativePath.split( air.File.separator ).pop();
			}
			this.store.newItem(itemData);
			// console.log("after newItem");
		},
		_loadPluginsInDir: function(/*air.File*/pluginsDir) {
			// walk the directory and peek inside sub-directories for our descriptor

			console.log("walking plugin directory: " + pluginsDir.nativePath);
			dojo.forEach(
				dojo.filter(
					pluginsDir.getDirectoryListing(), 
					function(file) {
						return file.isDirectory;
					}
				), 
				function(dir) {
					// console.log("looking in " + dir.nativePath + " for " + this.propertiesFilename);
					var descriptorFile = dir.resolvePath( this.propertiesFilename );
					console.log(descriptorFile.nativePath + " exists? " + descriptorFile.exists);
					if(descriptorFile.exists) {
						var json = toolbox.utils.readFile(descriptorFile);
						var descriptor = dojo.fromJson(json);
						// modules can be disabled
						if(descriptor.disabled){
							return;
						}
						// console.log(json);
						this._loadPlugin(dir, descriptor);
					}
				}, 
				this
			);
		},

		getInfo: function() {
			console.log("not implemented");
		}

	});
	console.log("calling init");
	moduleManager.init();

})();


