dojo.provide("toolbox.utils");

toolbox.utils.isLoaded = true;
dojo.mixin(toolbox.utils, {
	pathSlash: air.File.separator,
	storageDir: air.File.applicationStorageDirectory,

	mergeDirectories: function(source, target, excludeRegExp) {
		if(!excludeRegExp) {
			// default filter is to exclude dot files/directories
			excludeRegExp = /^\./; 
		}
		if(typeof excludeRegExp == "string") {
			excludeRegExp = new RegExp(excludeRegExp);
		}
		var getName = function(path, delim) {
			var tokens = path.split(delim || "/"); 
			var last = tokens.pop();
			if(!last.length) {
				// path ended in "/"
				last = tokens.pop();
			}
			return last;
		}; 
		// console.log("merging " + source.nativePath + " into " + target.nativePath);
		dojo.forEach(source.getDirectoryListing(), function(sourceFile) {
			var name = sourceFile.name; 
			if(excludeRegExp.test(name)) {
				return;
			}
			var targetFile = target.resolvePath(name); 
			if(sourceFile.isDirectory) {
				// copy directory
				if(targetFile.exists) {
					// recurse to avoid clobbering right-hand files unintentionally
					toolbox.utils.mergeDirectories(sourceFile, targetFile, excludeRegExp); 
				} else {
					if(!targetFile.parent.exists){
						targetFile.parent.createDirectory();
					}
					sourceFile.copyTo(targetFile);
				}
			} else {
				// copy file
				if(targetFile.exists) {
					// delete the target before copying
					// there's error handling we could do here..
					targetFile.deleteFile();
				}
				sourceFile.copyTo(targetFile);
			}
			// console.log("done copying " + name + " to " + targetFile.nativePath);
		}, this);
	},

	readFile: function(/* air.File | String */file, /* dojo.Deferred */deferred) {
		var isAsync = Boolean(deferred); 
		console.log("readFile, isAsync: " + isAsync);
		if(typeof file == "string") {
			file = new air.File(file);
		}
		if(!file.exists) {
			// TODO: could raise an error
			console.warn("couldnt readFile " + file.nativePath + " which doesnt exist");
			return "";
		}
		var stream = new air.FileStream();
		
		var readContents = function() {
			var btyeArray = new air.ByteArray();
			// readMultiByte will work for text files only??
			// var contentStr = stream.readMultiByte(file.size, air.File.systemCharset);
			stream.readBytes(btyeArray, 0, 0); // offset into byteArray, length (0 == all)
			return btyeArray.toString();
		}; 
		
		if(isAsync) {
			var fileCompleteHandler = function(evt) {
				var str = readContents();
				stream.removeEventListener(air.Event.COMPLETE, fileCompleteHandler);
				stream.close();
				deferred.callback(str);
			}
			var fileIOErrorHandler = function(evt) {
				stream.removeEventListener(air.Event.IOErrorEvent, fileIOErrorHandler);
				stream.close();
				console.warn("error opening " + file.nativePath);
				deferred.errback(evt);
			};
			stream.addEventListener(air.Event.COMPLETE, fileCompleteHandler);
			stream.addEventListener(air.IOErrorEvent, fileIOErrorHandler);
			
			console.log("readFile, openAsync");
			stream.openAsync(file, air.FileMode.READ);
			
			return deferred;
		} else {
			stream.open(file, air.FileMode.READ);
			var str = readContents();
			stream.close();
			return str;
		}
	},
	transformXML: function(xml,xsl)
	{
		var domParser = new DOMParser();
		var xmlObject = domParser.parseFromString(xml,"text/xml");
		var xslObject = domParser.parseFromString(xsl,"text/xml");
		var xsltProcessor = new XSLTProcessor();
		xsltProcessor.importStylesheet(xslObject);
		var result = xsltProcessor.transformToFragment(
			xmlObject,document);

		var serializer = new XMLSerializer();
		return serializer.serializeToString(result);
	}, 
	reCSS: function(doc) {
		if(!doc) {
			doc = dojo.doc;
		}
		var i,
		a,
		s;
		a = doc.getElementsByTagName('link');
		for (i = 0; i < a.length; i++) {
				s = a[i];
				if (s.rel.toLowerCase().indexOf('stylesheet') >= 0 && s.href) {
						var h = s.href.replace(/(&|%5C?)forceReload=\d+/, '');
						s.href = h + (h.indexOf('?') >= 0 ? '&': '?') + 'forceReload=' + (new Date().valueOf())
				}
		}
	},
	openURL: function(/* String */ url){
		/* Opens the URL in the user's default external web browser */
		var request = new air.URLRequest(url);
		air.navigateToURL(request);
	}
});



dojo.declare("toolbox.utils.Zip", null, {
	baseDir: toolbox.utils.storageDir,
	urlRequest: null,
	file: null,

	constructor: function( url ) {
		// console.log("in the Zip constructor, arg[0] is: " + url);
		
		if(url) {
			if(typeof url === "string") {
				if(/^(file|http|https|ftp|app-storage|app):/.test(url)) {
					url = new air.URLRequest(url);
				} else {
					// treat as filename
					url = new air.File(url);
				}
			}
			if(url.nativePath) {
				this.setFile(url);
			} else {
				this.setUrl(url);
			}
		}
		var fzip = this._fzip = new window.runtime.deng.fzip.FZip();
		fzip.addEventListener("complete", dojo.hitch(this, this._onLoadComplete));
		fzip.addEventListener("ioError", dojo.hitch(this, this._onError));
		fzip.addEventListener("fileLoaded", dojo.hitch(this, this._onFileLoaded));
		fzip.addEventListener("progress", dojo.hitch(this, this._onProgress));
	}, 
	setUrl: function(url) {
		// console.log("Zip.setUrl");
		if(typeof url == "string") {
			url = new air.URLRequest(url); 
		}
		this.urlRequest = url;
		return this.urlRequest;
	},
	setFile: function(file) {
		// console.log("Zip.setFile, typeof air.File:" + typeof air.File);
		if(typeof file == "string") {
			file = air.File.applicationDirectory.resolvePath(file);
		}
		this.urlRequest = new air.URLRequest(file.url);
	},
	setOutputDir: function(dir) {
		if(typeof dir == "string") {
			dir = new air.File(dir);
		}
		this.baseDir = dir; 
	
		// console.log("Zip.setOutputDir, dir is: " +this.baseDir.nativePath);
		return this.baseDir;
	},
	expand: function(dir) {
		// console.log("Zip.expand");
	
		this._fzip.load( this.urlRequest );
	},
	expandTo: function(dir) {
		this.setOutputDir(dir);
		this.expand();
	},
	_onError: function(err) {
		this.onError(err);
	},
	onError: function(err) {
		// stub
		console.warn("Zip.onError: " + err.type + ":" + err.text);
	},
	_onProgress: function(evt){
		this.onProgress(evt);
	},
	onProgress: function(evt){
		// stub
	},
	_onFileLoaded: function(evt){
		// create an artificial event instead of the awkward FZipEvent	
		this.fileCount = this._fzip.getFileCount()
		this.onFileLoaded({
			file: evt.file,
			fileCount: this.fileCount
		});
	},
	onFileLoaded: function(evt){
		// stub
		// evt.file is the filename
		// evt.fileCount is count of all files in the zip so far
		// as the file is streaming over, we cant get a total count until download is finished
	},
	_onLoadComplete: function(evt) {
		this.onLoadComplete(evt);
	},
	onLoadComplete: function(evt) {
		console.trace("caught the onComplete event");
	
		var fzip = this._fzip, 
			files = [],
			fileCount = fzip.getFileCount();
		console.trace("has files: " + fileCount);
	
		var zfile,
			dotRe = /^\.|\/\./,
			fileRe = /[^\\\/]\.[a-zA-Z0-9]+$/;
		
		for(var i=0; i<fileCount; i++) {
			zfile = fzip.getFileAt(i);
			// skip . files, directories (e.g. .svn)
			if(dotRe.test(zfile.filename.toString())) {
				continue;
			}
			// skip directories, files without extension
			if(fileRe.test(zfile.filename.toString())) {
				files.push(zfile);
			} else {
				// console.log("skipping non-file: " + zfile.filename);
			}
		}
		// air.File.applicationStorageDirectory
		dojo.forEach(files, function(zfile) {
			this._writeOutFile(zfile);
			// console.log("good filename: " + zfile.filename);
		}, this); 
	},
	_writeOutFile: function(zFile) {
		var baseDir = this.baseDir,
			pathSlash = toolbox.utils.pathSlash,
			filename = zFile.filename,
			pathname = [baseDir.nativePath, filename].join(pathSlash);
	
		var file = new air.File(pathname);
		if(!file.parent.exists) {
			file.parent.createDirectory(); 
			// toolbox.utils.createIntercedingDirectories(baseDir, file);
		}
	
		var fs = new air.FileStream();
		fs.open(file, air.FileMode.WRITE);
		// TODO: catch exceptions
		fs.writeBytes( zFile.content );
		fs.close();
	}
});
